#!/usr/bin/env python3
"""
Module PYSIDEEDITCTRL -- Python PySide Editing Widgets
Sub-Package GUI.TOOLKITS.PYSIDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PySide GUI objects for edit controls.
"""

from PySide import QtGui as qt

from plib.gui.defs import *
from plib.gui._widgets import editctrl

from ._pysidecommon import _PQtWidgetMeta, _PQtWidget, _PQtClientWidget


class _PQtEditMixin(object):
    
    fn_get_readonly = 'isReadOnly'
    fn_set_readonly = 'setReadOnly'


class PQtEditBox(_PQtEditMixin, _PQtWidget, qt.QLineEdit,
                 editctrl.PEditBoxBase, metaclass=_PQtWidgetMeta):
    
    fn_get_text = 'str_text'
    fn_set_text = 'setText'
    
    def __init__(self, parent, target=None, geometry=None, expand=True):
        qt.QLineEdit.__init__(self, parent)
        if expand:
            self._horiz = qt.QSizePolicy.MinimumExpanding
        else:
            self._horiz = qt.QSizePolicy.Fixed
        self.setSizePolicy(self._horiz, qt.QSizePolicy.Fixed)
        editctrl.PEditBoxBase.__init__(self, target, geometry)
    
    def str_text(self):
        return str(self.text())
    
    def set_width(self, width):
        self.resize(width, self.height())
        # Qt line edits don't seem to fully respect qt.QSizePolicy.Fixed
        if self._horiz == qt.QSizePolicy.Fixed:
            self.setMaximumWidth(width)
        elif self._horiz == qt.QSizePolicy.MinimumExpanding:
            self.setMinimumWidth(width)


class PQtEditControl(_PQtEditMixin, _PQtClientWidget, qt.QTextEdit,
                     editctrl.PEditControlBase, metaclass=_PQtWidgetMeta):
    
    fn_get_text = 'toPlainText'
    fn_set_text = 'setPlainText'
    
    event_signals = (SIGNAL_TEXTSTATECHANGED,)
    
    def __init__(self, parent, target=None, geometry=None, scrolling=False):
        # Flags for tracking state
        self._undoflag = False
        self._redoflag = False
        self._clipflag = False
        
        qt.QTextEdit.__init__(self, parent)
        self.setAcceptRichText(False)
        if scrolling:
            self.setLineWrapMode(qt.QTextEdit.NoWrap)
        editctrl.PEditControlBase.__init__(self, target, geometry)
        
        # Signal connections for tracking state
        statesigs = [
            ("undo", self._check_undoflag),
            ("redo", self._check_redoflag),
            ("copy", self._check_clipflag)
        ]
        for signame, target in statesigs:
            getattr(self, "{}Available".format(signame)).connect(target)
    
    def set_width(self, width):
        self.resize(width, self.height())
        # Qt text edits don't seem to fully respect
        # qt.QSizePolicy.MinimumExpanding
        self.setMinimumWidth(width)
    
    def set_height(self, height):
        self.resize(self.width(), height)
        # Qt text edits don't seem to fully respect
        # qt.QSizePolicy.MinimumExpanding
        self.setMinimumHeight(height)
    
    def textStateChanged(self):
        self.do_notify(SIGNAL_TEXTSTATECHANGED)
    
    def _check_undoflag(self, available):
        self._undoflag = available
        self.textStateChanged()
    
    def _check_redoflag(self, available):
        self._redoflag = available
        self.textStateChanged()
    
    def _check_clipflag(self, available):
        self._clipflag = available
        self.textStateChanged()
    
    def can_undo(self):
        return self.isUndoRedoEnabled() and self._undoflag
    
    def can_redo(self):
        return self.isUndoRedoEnabled() and self._redoflag
    
    def can_clip(self):
        return self._clipflag
    
    def can_paste(self):
        return self.canPaste()
    
    def clear_edit(self):
        self.clear()
    
    def undo_last(self):
        self.undo()
    
    def redo_last(self):
        self.redo()
    
    def select_all(self):
        self.selectAll()
    
    def delete_selected(self):
        pass  # FIXME
    
    def copy_to_clipboard(self):
        self.copy()
    
    def cut_to_clipboard(self):
        self.cut()
    
    def paste_from_clipboard(self):
        self.paste()



#!/usr/bin/env python3
"""
Module WXCOMMON -- Python wxWidgets Common Objects
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI common objects.
"""

import string

import wx
import wx.grid
from wx.lib.evtmgr import eventManager
from wx.lib.newevent import NewEvent

from plib.gui.defs import *

# Style constant for edit controls that are scrolling

_wx_ec_scroll_style = wx.VSCROLL | wx.HSCROLL | wx.TE_DONTWRAP

# FIXME: These are only good for buttons and menus, and the toolbar ones
# in wx.ArtProvider don't match all the ids! WTF!@#$%^&*()
_wxstockids = {
    ACTION_FILENEW: wx.ID_NEW,
    ACTION_FILEOPEN: wx.ID_OPEN,
    ACTION_FILESAVE: wx.ID_SAVE,
    ACTION_FILESAVEAS: wx.ID_SAVEAS,
    ACTION_FILECLOSE: wx.ID_CLOSE,
    ACTION_EDIT: wx.ID_EDIT,
    ACTION_EDITUNDO: wx.ID_UNDO,
    ACTION_EDITREDO: wx.ID_REDO,
    ACTION_EDITCUT: wx.ID_CUT,
    ACTION_EDITCOPY: wx.ID_COPY,
    ACTION_EDITPASTE: wx.ID_PASTE,
    ACTION_EDITDELETE: wx.ID_DELETE,
    ACTION_EDITSELECTALL: wx.ID_SELECTALL,
    ACTION_REFRESH: wx.ID_REFRESH,
    ACTION_ADD: wx.ID_ADD,
    ACTION_REMOVE: wx.ID_REMOVE,
    ACTION_APPLY: wx.ID_APPLY,
    ACTION_OK: wx.ID_OK,
    ACTION_CANCEL: wx.ID_CANCEL,
    ACTION_PREFS: wx.ID_PREFERENCES,
    ACTION_ABOUT: wx.ID_ABOUT,
    ACTION_EXIT: wx.ID_EXIT
}

_wxalignmap = {
    ALIGN_LEFT: wx.LIST_FORMAT_LEFT,
    ALIGN_CENTER: wx.LIST_FORMAT_CENTRE,
    ALIGN_RIGHT: wx.LIST_FORMAT_RIGHT
}

_wxicons = {
    MBOX_INFO: wx.ICON_INFORMATION,
    MBOX_WARN: wx.ICON_EXCLAMATION,
    MBOX_ERROR: wx.ICON_ERROR,
    MBOX_QUERY: wx.ICON_QUESTION
}

_wxfontfamilies = {
    wx.FONTFAMILY_ROMAN: ["Times New Roman"],
    wx.FONTFAMILY_SWISS: ["Arial", "Verdana"],
    wx.FONTFAMILY_MODERN: ["Courier New"]
}

# Define our own custom events
ClosingEvent, EVT_CLOSING = NewEvent()
ShownEvent, EVT_SHOWN = NewEvent()
HiddenEvent, EVT_HIDDEN = NewEvent()
TextStateEvent, EVT_TEXT_STATE_CHANGED = NewEvent()
#CellChangedEvent, EVT_CELL_CHANGED = NewEvent()
ToggledEvent, EVT_CHECK_TOGGLED = NewEvent()

_wxcustommap = {
    SIGNAL_TOGGLED: ToggledEvent,
    #SIGNAL_CELLCHANGED: CellChangedEvent,
    SIGNAL_TEXTSTATECHANGED: TextStateEvent,
    SIGNAL_CLOSING: ClosingEvent,
    SIGNAL_SHOWN: ShownEvent,
    SIGNAL_HIDDEN: HiddenEvent
}

_wxeventmap = {
    SIGNAL_ACTIVATED: wx.EVT_MENU,
    SIGNAL_CLICKED: wx.EVT_BUTTON,
    SIGNAL_CHECKTOGGLED: wx.EVT_CHECKBOX,
    SIGNAL_TOGGLED: EVT_CHECK_TOGGLED,
    SIGNAL_SELECTED: wx.EVT_COMBOBOX,
    SIGNAL_FOCUS_IN: wx.EVT_SET_FOCUS,
    SIGNAL_FOCUS_OUT: wx.EVT_KILL_FOCUS,
    SIGNAL_LISTSELECTED: wx.EVT_TREE_SEL_CHANGED,
    SIGNAL_CELLSELECTED: wx.grid.EVT_GRID_SELECT_CELL,
    #SIGNAL_TABLECHANGED: wx.grid.EVT_GRID_CELL_CHANGE,
    SIGNAL_CELLCHANGED: wx.grid.EVT_GRID_CELL_CHANGE,
    SIGNAL_TEXTCHANGED: wx.EVT_TEXT,
    SIGNAL_TEXTSTATECHANGED: EVT_TEXT_STATE_CHANGED,
    SIGNAL_EDITCHANGED: wx.EVT_TEXT,
    SIGNAL_ENTER: wx.EVT_TEXT_ENTER,
    SIGNAL_TABCHANGED: wx.EVT_NOTEBOOK_PAGE_CHANGED,
    SIGNAL_CLOSING: EVT_CLOSING,
    SIGNAL_SHOWEVENT: wx.EVT_SHOW,
    SIGNAL_SHOWN: EVT_SHOWN,
    SIGNAL_HIDDEN: EVT_HIDDEN,
    SIGNAL_QUERYCLOSE: wx.EVT_CLOSE,
    SIGNAL_BEFOREQUIT: wx.EVT_WINDOW_DESTROY
}


# 'Wrapper' functions for certain events to repackage parameters

def wx_plain_wrapper(self, target):
    def wrapper(event):
        target()
    return wrapper


def wx_toggled_wrapper(self, target):
    def wrapper(event):
        target(self.GetValue())
    return wrapper


def wx_selected_wrapper(self, target):
    def wrapper(event):
        target(self.GetSelection())
    return wrapper


def wx_listviewchanged_wrapper(self, target):
    def wrapper(event):
        item = self.GetItemPyData(event.GetItem())
        # Hack to filter out events fired too soon by some Wx versions
        if item is not None:
            target(item)
    return wrapper


def wx_cellselected_wrapper(self, target):
    def wrapper(event):
        # FIXME: As noted in scrips-edit.py, we have to have hacks
        # in user code to work around the fact that wx fires this
        # event and the table changed event out of sequence; when
        # you press Enter to finish a cell edit, wx in its infinite
        # wisdom moves the cell cursor to the next cell down, but it
        # does *NOT* fire the table changed event *before* moving the
        # cursor; no, it first moves the cursor *AND FIRES THE CELL
        # SELECTED EVENT*, and THEN it *FIRES THE TABLE CHANGED EVENT
        # FOR THE CELL THAT IS NO LONGER SELECTED BECAUSE IT MOVED
        # THE CURSOR BEFORE TELLING USER CODE THAT THE PREVIOUS CELL
        # HAD BEEN CHANGED* @#$%^&*! Anyway, at some point I may
        # figure out a way to have the library code here fix this
        # f**ked event ordering, so user code sees an API that makes
        # some kind of f**king sense.
        target(event.GetRow(), event.GetCol())
        event.Skip(True)
    return wrapper


def wx_tablechanged_wrapper(self, target):
    def wrapper(event):
        target(event.GetRow(), event.GetCol())
    return wrapper


def wx_tabchanged_wrapper(self, target):
    def wrapper(event):
        target(event.GetSelection())
    return wrapper


_wxwrappermap = {
    SIGNAL_ACTIVATED: wx_plain_wrapper,
    SIGNAL_CLICKED: wx_plain_wrapper,
    SIGNAL_TOGGLED: wx_toggled_wrapper,
    #SIGNAL_CHECKTOGGLED: wx_plain_wrapper,
    SIGNAL_SELECTED: wx_selected_wrapper,
    SIGNAL_FOCUS_IN: wx_plain_wrapper,
    SIGNAL_FOCUS_OUT: wx_plain_wrapper,
    SIGNAL_LISTSELECTED: wx_listviewchanged_wrapper,
    SIGNAL_CELLSELECTED: wx_cellselected_wrapper,
    #SIGNAL_TABLECHANGED: wx_tablechanged_wrapper,
    SIGNAL_CELLCHANGED: wx_tablechanged_wrapper,
    SIGNAL_TEXTCHANGED: wx_plain_wrapper,
    SIGNAL_EDITCHANGED: wx_plain_wrapper,
    SIGNAL_ENTER: wx_plain_wrapper,
    SIGNAL_TABCHANGED: wx_tabchanged_wrapper,
    SIGNAL_CLOSING: wx_plain_wrapper,
    SIGNAL_SHOWN: wx_plain_wrapper,
    SIGNAL_HIDDEN: wx_plain_wrapper
}


class _PWxCommunicator(object):
    """Mixin class to abstract notification functionality in wxWidgets.
    """
    
    def setup_notify(self, signal, target, wrap=True):
        if signal in _wxeventmap:
            event = _wxeventmap[signal]
            if wrap and (signal in _wxwrappermap):
                handler = _wxwrappermap[signal](self, target)
            else:
                handler = target
            
            # Do the following instead of self.Bind(event, handler) so
            # that multiple handlers can receive a single event
            eventManager.Register(handler, event, self)
    
    def do_notify(self, signal, *args):
        if signal in _wxcustommap:
            event = _wxcustommap[signal]()
        elif signal in _wxeventmap:
            event = wx.Event()
            event.SetEventType(_wxeventmap[signal])
            if hasattr(self, 'id'):
                event.SetEventID(self.id)
            elif hasattr(self, '_id'):
                event.SetEventID(self._id)
            elif hasattr(self, 'GetId'):
                event.SetEventID(self.GetId())
        if event is not None:
            event.SetEventObject(self)
            self.AddPendingEvent(event)


_wx_font_weights = [wx.FONTWEIGHT_NORMAL, wx.FONTWEIGHT_BOLD]
_wx_font_styles = [wx.FONTSTYLE_NORMAL, wx.FONTSTYLE_ITALIC]


class _PWxWidgetBase(object):
    """Mixin class to provide basic wx widget methods.
    """
    
    fn_enable_get = 'IsEnabled'
    fn_enable_set = 'Enable'
    
    _depth_test_str = string.ascii_letters + string.digits
    _depth_scale_factor = 1.3
    
    def update_widget(self):
        self.Refresh()
    
    def preferred_width(self):
        return self.GetSizeTuple()[0]
    
    def preferred_height(self):
        return self.GetSizeTuple()[1]
    
    def set_width(self, width):
        height = self.GetSizeTuple()[1]
        self.SetSizeWH(width, height)
        self.SetMinSize((width, height))
    
    def set_height(self, height):
        width = self.GetSizeTuple()[0]
        self.SetSizeWH(width, height)
        self.SetMinSize((width, height))
    
    def set_position(self, left, top):
        if not (None in (left, top)):
            self.SetPosition(wx.Point(left, top))
    
    def set_foreground_color(self, color):
        self.SetForegroundColour(color)
    
    def set_background_color(self, color):
        self.SetBackgroundColour(color)
    
    def get_font_name(self):
        return self.GetFont().GetFaceName()
    
    def get_font_size(self):
        return self.GetFont().GetPointSize()
    
    def get_font_bold(self):
        return (self.GetFont().GetWeight() == _wx_font_weights[1])
    
    def get_font_italic(self):
        return (self.GetFont().GetStyle() == _wx_font_styles[1])
    
    def _wx_font_object(self, font_name, font_size, bold, italic):
        font_family = wx.FONTFAMILY_DEFAULT
        for family, names in _wxfontfamilies.items():
            if font_name in names:
                font_family = family
                break
        font_style = _wx_font_styles[int(italic)]
        font_weight = _wx_font_weights[int(bold)]
        font = wx.Font(font_size, font_family, font_style, font_weight)
        font.SetFaceName(font_name)
        return font
    
    def _set_font_object(self, font_name, font_size, bold, italic):
        font = self._wx_font_object(font_name, font_size, bold, italic)
        if hasattr(self, '_depth_w'):
            # Hack to make list view column auto-sizing work correctly
            # on font change
            old_extent = self.GetTextExtent(self._depth_test_str)[0]
            new_extent = self.GetFullTextExtent(self._depth_test_str, font)[0]
            self._depth_w = int(
                self._depth_w * self._depth_scale_factor * new_extent /
                old_extent)
        self.SetFont(font)
        if hasattr(self, 'SetDefaultCellFont'):
            self.SetDefaultCellFont(font)  # takes care of the table widget
        if hasattr(self, 'AutoSizeCols'):
            self.AutoSizeCols()  # this takes care of the list view widget
    
    def set_focus(self):
        self.SetFocus()


class _PWxWidget(_PWxCommunicator, _PWxWidgetBase):
    """Mixin class for wx widgets that can send/receive signals.
    """
    pass


class _PWxClientWidget(_PWxWidget):
    """Mixin class for wx main window client widgets.
    """
    pass



#!/usr/bin/env python3
"""
Module HEADER -- Table/List Header Classes
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines header classes for tables and list views. The
following classes are defined:

- PHeaderLabel: a label for the header of a table or
  list/treeview widget.
"""

from plib.gui.defs import *


class PHeaderLabel(object):
    """Encapsulates a header label for a table or list/tree view.
    """
    
    def __init__(self, text, width=-1, align=ALIGN_LEFT, readonly=False):
        self.text = text
        self.width = width
        self.align = align
        self.readonly = readonly
    
    def __str__(self):
        return self.text



#!/usr/bin/env python3
"""
Module WXPANEL -- Python wxWidgets Panel Objects
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI objects for the panel widgets.
"""

import wx

from plib.gui.defs import *

from plib.gui._widgets import panel

from ._wxcommon import _PWxWidgetBase

_wxstyles = {
    PANEL_NONE: wx.NO_BORDER,
    PANEL_BOX: wx.SIMPLE_BORDER,
    PANEL_RAISED: wx.RAISED_BORDER,
    PANEL_SUNKEN: wx.SUNKEN_BORDER
}

_wxlayouts = {
    LAYOUT_NONE: wx.HORIZONTAL,
    LAYOUT_HORIZONTAL: wx.HORIZONTAL,
    LAYOUT_VERTICAL: wx.VERTICAL
}

_wxflags = {
    ALIGN_LEFT: wx.ALIGN_LEFT,
    ALIGN_RIGHT: wx.ALIGN_RIGHT,
    ALIGN_TOP: wx.ALIGN_TOP,
    ALIGN_BOTTOM: wx.ALIGN_BOTTOM
}


class PWxPanel(_PWxWidgetBase, wx.Panel, panel.PPanelBase):
    
    def __init__(self, parent,
                 layout=LAYOUT_NONE, style=PANEL_NONE, align=ALIGN_JUST,
                 margin=-1, spacing=-1, width=-1, height=-1):
        
        wx.Panel.__init__(self, parent, style=_wxstyles[style])
        self._margin = margin
        self._spacing = spacing
        self._expand = True
        self._haswidgets = False
        self._sizer = wx.BoxSizer(_wxlayouts[layout])
        panel.PPanelBase.__init__(self, parent, layout, style, align,
                                  margin, spacing, width, height)
    
    def _addwidget(self, widget):
        # People say the wxWidgets layout model is simple and easy to use,
        # but comparing this cruft with the Qt or KDE code, I'm not so sure...
        
        if self._haswidgets:
            if self._spacing > -1:
                self._sizer.AddSpacer(self._spacing)
        elif self._margin > -1:
            self._sizer.AddSpacer(self._margin)
        
        if hasattr(widget, '_align') and (widget._align != ALIGN_JUST):
            proportion = 0
            flag = _wxflags[widget._align]
        else:
            proportion = 1
            flag = 0
        
        if hasattr(widget, '_expand') and widget._expand:
            flag |= wx.EXPAND
        
        if self._sizer.GetOrientation() == wx.VERTICAL:
            horiz_pad = self._margin
            vert_pad = -1
            flag |= wx.ALIGN_CENTER_HORIZONTAL
        else:
            horiz_pad = -1
            vert_pad = self._margin
            flag |= wx.ALIGN_CENTER_VERTICAL
        
        if horiz_pad > -1:
            flag |= wx.LEFT | wx.RIGHT
            border = horiz_pad
        elif vert_pad > -1:
            flag |= wx.TOP | wx.BOTTOM
            border = vert_pad
        else:
            border = 0
        
        # At last we can actually do what we came for...
        self._sizer.Add(widget, proportion, flag, border)
        if not self._haswidgets:
            self._haswidgets = True
    
    def _dolayout(self):
        if self._margin > -1:
            self._sizer.AddSpacer(self._margin)
        self.SetSizerAndFit(self._sizer)
    
    def set_min_size(self, width, height):
        curr_w, curr_h = self._sizer.GetMinSize().Get()
        if width < 0:
            width = curr_w
        if height < 0:
            height = curr_h
        self._sizer.SetMinSize((width, height))
    
    def set_box_width(self, width):
        pass  # FIXME
    
    def set_margin(self, margin):
        # Margin is dealt with differently in wxWidgets, see above
        pass
    
    def set_spacing(self, spacing):
        # Spacing is dealt with differently in wxWidgets, see above
        pass



#!/usr/bin/env python3
"""
Module KDE4ACTION -- Python KDE Action Objects
Sub-Package GUI.TOOLKITS.KDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the KDE 4 GUI objects for handling user actions.
"""

from PyQt4 import Qt as qt
from PyKDE4 import kdeui

from plib.gui._base import action

from ._kde4common import (_PKDECommunicator, _PKDEWidgetBase,
                         _kdestandardicons)


class PKDEMenu(_PKDEWidgetBase, kdeui.KMenuBar, action.PMenuBase):
    """Customized KDE menu class.
    """
    
    popupclass = kdeui.KMenu
    
    def __init__(self, mainwidget):
        kdeui.KMenuBar.__init__(self, mainwidget)
        action.PMenuBase.__init__(self, mainwidget)
        mainwidget.setMenuBar(self)
    
    def _add_popup(self, title, popup):
        popup.setTitle(title)
        self.addMenu(popup)
    
    def _add_popup_action(self, act, popup):
        popup.addAction(act)


class PKDEToolBar(_PKDEWidgetBase, kdeui.KToolBar, action.PToolBarBase):
    """Customized KDE toolbar class.
    """
    
    def __init__(self, mainwidget):
        kdeui.KToolBar.__init__(self, mainwidget)
        if mainwidget.large_icons:
            sz = self.iconSize()
            self.setIconSize(qt.QSize(
                *[i * 3 / 2 for i in (sz.width(), sz.height())]))
        if mainwidget.show_labels:
            style = qt.Qt.ToolButtonTextUnderIcon
        else:
            style = qt.Qt.ToolButtonIconOnly
        self.setToolButtonStyle(style)
        action.PToolBarBase.__init__(self, mainwidget)
        mainwidget.addToolBar(self)
    
    def add_action(self, act):
        self.addAction(act)
    
    def add_separator(self):
        self.addSeparator()


class PKDEAction(_PKDECommunicator, kdeui.KAction, action.PActionBase):
    """Customized KDE action class.
    """
    
    def __init__(self, key, mainwidget):
        kdeui.KAction.__init__(self, mainwidget)
        # Some actions are better handled by retrieving the standard
        # icon but not using the standard action, which sets other
        # stuff differently than we want (e.g., the action text)
        if key in _kdestandardicons:
            icon = _kdestandardicons[key]
        else:
            icon = qt.QIcon(qt.QPixmap(
                self.get_icon_filename(key)))
        self.setIcon(kdeui.KIcon(icon))
        self.setText(qt.QString(self.get_menu_str(key)))
        self.setToolTip(qt.QString(self.get_toolbar_str(key)))
        s = self.get_accel_str(key)
        if s is not None:
            self.setShortcut(qt.QKeySequence(s))
        action.PActionBase.__init__(self, key, mainwidget)
    
    def enable(self, enabled):
        self.setEnabled(enabled)



#!/usr/bin/env python3
"""
Module ACTION -- GUI Action Classes
Sub-Package GUI.BASE of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the classes that handle user actions.
"""

from plib.gui.defs import *
from plib.gui.common import *

from plib.gui._base import _notify


class PActionBase(_notify._PNotifyBase):
    """Base class for GUI action objects.
    
    Uses ``plib.gui.common`` globals to determine action pixmaps and labels;
    the globals can be modified as long as it's done before any actions are
    instantiated (if the actions are being created by a PMainWindow, which
    is the intended use case, the globals must be modified either before
    PMainWindow.__init__ is called (because the actions are created inside
    the constructor), or inside an overridden PMainWindow.createactions.
    """
    
    signal = SIGNAL_ACTIVATED
    
    def __init__(self, key, mainwidget):
        self.init_setup(key, mainwidget)
    
    def init_setup(self, key, mainwidget):
        # Convenience method so standard actions can use it too
        self.key = key
        self.mainwin = mainwidget
        
        # Add ourself to main menu and toolbar if they exist
        if mainwidget.menu is not None:
            mainwidget.menu.add_action(self)
        if mainwidget.toolbar is not None:
            mainwidget.toolbar.add_action(self)
    
    def enable(self, enabled):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError


class _PGUIBase(object):
    """Base class for GUI menus and toolbars to add actions.
    """
    
    def __init__(self, mainwidget):
        self.mainwin = mainwidget
    
    def add_action(self, action):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError


class PMenuBase(_PGUIBase):
    """Base class for GUI menu.
    """
    
    popupclass = None
    
    def __init__(self, mainwidget):
        _PGUIBase.__init__(self, mainwidget)
        self.popups = {}
    
    def _add_popup(self, title, popup):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def add_popup(self, title):
        if self.popupclass is not None:
            popup = self.popupclass(self.mainwin)
            self._add_popup(title, popup)
            self.popups[title] = popup
    
    def get_popup(self, title):
        if title in self.popups:
            return self.popups[title]
        else:
            return None
    
    def _add_popup_action(self, action, popup):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def add_action(self, action):
        for title, items in menukeygroups:
            if action.key in items:
                if self.get_popup(title) is None:
                    self.add_popup(title)
                self._add_popup_action(action, self.popups[title])


class PToolBarBase(_PGUIBase):
    """Base class for GUI toolbar.
    """
    
    def add_separator(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError



#!/usr/bin/env python3
"""
GUI-DISPLAY.PY
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

A useless demo app that displays all of the available
GUI actions and their associated images.
"""

import sys

from plib.gui import __version__
from plib.gui import main as gui
from plib.gui import defs


class GUIDisplayWindow(gui.PMainWindow):
    
    aboutdata = {
        'name': "GUIDisplay",
        'version': "{} on Python {}".format(
            __version__,
            sys.version.split()[0]
        ),
        'copyright': "Copyright (C) 2008-2013 by Peter A. Donis",
        'license': "GNU General Public License (GPL) Version 2",
        'description': "GUI Display Demo",
        'developers': ["Peter Donis"],
        'website': "http://www.peterdonis.net"
    }
    
    actionflags = sorted(
        getattr(defs, name)
        for name in dir(defs)
        if name.startswith('ACTION_')
    )
    
    clientwidgetclass = gui.PTextDisplay
    defaultcaption = "GUI Display Demo"
    placement = (defs.SIZE_MAXIMIZED, defs.MOVE_NONE)
    
    def __init__(self, app):
        gui.PMainWindow.__init__(self, app)
        self.clientwidget.set_text("PLIB3 GUI Display Demo Application.")


def run():
    from plib.stdlib.options import parse_options
    optlist = (
        ("-l", "--large-icons", {
            'action': "store_true",
            'dest': "large_icons", 'default': False,
            'help': "Use large toolbar icons"
        }),
        ("-s", "--show-labels", {
            'action': "store_true",
            'dest': "show_labels", 'default': False,
            'help': "Show toolbar button labels"
        }),
        ("-q", "--query-on-exit", {
            'action': "store_true",
            'dest': "queryonexit", 'default': False,
            'help': "Ask for confirmation on app exit"
        })
    )
    opts, args = parse_options(optlist)
    # The options object supports a dictionary interface,
    # making it easy to update class fields from it
    for opt, value in opts.items():
        setattr(GUIDisplayWindow, opt, value)
    gui.runapp(GUIDisplayWindow)


if __name__ == '__main__':
    run()



#!/usr/bin/env python3
"""
Module WXBUTTON -- Python wxWidgets Button Widgets
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI objects for button widgets.
"""

import wx
#import wx.lib.buttons

from plib.gui.defs import *
from plib.gui._widgets import button

from ._wxcommon import _PWxWidget, _wxstockids


class _PWxButtonMixin(object):
    
    def set_caption(self, caption):
        self.SetLabel(caption)
    
    def set_icon(self, pxname):
        # No way currently to set custom icons in wxWidgets (at least,
        # none that are worth considering here...)
        pass


class PWxButton(_PWxButtonMixin, _PWxWidget, wx.Button, button.PButtonBase):
    
    def __init__(self, parent, caption=None, pxname=None,
                 target=None, geometry=None):
        
        _id = -1
        # We can't wait until PButtonBase.__init__ to do this in wxWidgets
        if isinstance(caption, int):
            # 'caption' is actually an action key
            if caption in _wxstockids:
                _id = _wxstockids[caption]
                caption = None
        wx.Button.__init__(self, parent, _id)
        self._align = ALIGN_LEFT  # used by PWxPanel to determine placement
        button.PButtonBase.__init__(self, caption, pxname, target, geometry)


class PWxCheckBox(_PWxButtonMixin, _PWxWidget, wx.CheckBox,
                  button.PCheckBoxBase):
    
    fn_get_checked = 'GetValue'
    fn_set_checked = 'SetValue'
    
    def __init__(self, parent, caption=None, pxname=None, tristate=False,
                 target=None, geometry=None):
        
        if tristate:
            style = wx.CHK_3STATE
        else:
            style = wx.CHK_2STATE
        wx.CheckBox.__init__(self, parent, style=style)
        self._align = ALIGN_LEFT  # used by PWxPanel to determine placement
        button.PCheckBoxBase.__init__(self, caption, pxname, tristate,
                                      target, geometry)
        
        # Pass our current checked state in the visible SIGNAL_TOGGLED signal
        self.setup_notify(SIGNAL_CHECKTOGGLED, self.OnCheckToggled)
    
    def OnCheckToggled(self, event):
        # The event wrapper will fill in our checked state
        self.do_notify(SIGNAL_TOGGLED)
    
    def make_tristate(self):
        # This is done in the constructor in wxWidgets
        pass



#!/usr/bin/env python3
"""
Module COMMON -- Python GUI Common Global Objects
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains common global objects for
the GUI sub-package. These are provided in a separate
module so that, if desired, these objects can be
mutated prior to instantiating your GUI application
(that is, prior to calling gui.runapp() to create
your application, or instantiating your own GUI
application class manually). These objects should
not be mutated after your application is running
because they are used on initialization to create
menu items, toolbar buttons, and actions, and after
that they must remain unchanged.
"""

import sys
import os
from functools import partial

from plib.gui import __path__ as guipath
from plib.gui.defs import *
from plib.gui._toolkit import toolkit

# list of 'automagically connected' signals (meaning that to catch
# them you have to override a method instead of calling setup_notify
# to have the signal sent to you)
automagic_signals = [SIGNAL_QUERYCLOSE, SIGNAL_BEFOREQUIT]

# useful combinations of action constants
ACTIONS_FILE = [
    ACTION_FILENEW, ACTION_FILEOPEN, ACTION_FILESAVE, ACTION_FILESAVEAS,
    ACTION_FILECLOSE]
ACTIONS_FILEMENU = ACTIONS_FILE + [ACTION_EXIT]
ACTIONS_EDIT = [
    ACTION_EDITUNDO, ACTION_EDITREDO,
    ACTION_EDITCUT, ACTION_EDITCOPY, ACTION_EDITPASTE,
    ACTION_EDITDELETE, ACTION_EDITSELECTALL]
ACTIONS_ACTION = [
    ACTION_VIEW, ACTION_EDIT,
    ACTION_OK, ACTION_CANCEL,
    ACTION_REFRESH, ACTION_ADD, ACTION_REMOVE,
    ACTION_APPLY, ACTION_COMMIT, ACTION_ROLLBACK]
ACTIONS_OPTIONS = [ACTION_PREFS]
ACTIONS_OTHER = [
    ACTION_PREFS,
    ACTION_ABOUT, ACTION_ABOUTTOOLKIT,
    ACTION_EXIT]
ACTIONS_HELP = [ACTION_ABOUT, ACTION_ABOUTTOOLKIT]

# action dictionary
actiondict = {
    ACTION_FILENEW: ["filenew", "&New"],
    ACTION_FILEOPEN: ["fileopen", "&Open"],
    ACTION_FILESAVE: ["filesave", "&Save"],
    ACTION_FILESAVEAS: ["filesaveas", "Save &As"],
    ACTION_FILECLOSE: ["fileclose", "&Close"],
    ACTION_VIEW: ["view", "&View"],
    ACTION_EDIT: ["button_edit", "&Edit"],
    ACTION_EDITUNDO: ["editundo", "&Undo"],
    ACTION_EDITREDO: ["editredo", "&Redo"],
    ACTION_EDITCUT: ["editcut", "Cu&t"],
    ACTION_EDITCOPY: ["editcopy", "&Copy"],
    ACTION_EDITPASTE: ["editpaste", "&Paste"],
    ACTION_EDITDELETE: ["editdelete", "&Delete"],
    ACTION_EDITSELECTALL: ["selectall", "Se&lect All"],
    ACTION_REFRESH: ["button_refresh", "Re&fresh"],
    ACTION_ADD: ["button_add", "A&dd"],
    ACTION_REMOVE: ["button_remove", "Remo&ve"],
    ACTION_APPLY: ["apply", "Appl&y"],
    ACTION_COMMIT: ["commit", "Co&mmit"],
    ACTION_ROLLBACK: ["rollback", "Rollbac&k"],
    ACTION_OK: ["button_ok", "&Ok"],
    ACTION_CANCEL: ["button_cancel", "&Cancel"],
    ACTION_PREFS: ["prefs", "&Preferences..."],
    ACTION_ABOUT: ["about", "A&bout..."],
    ACTION_ABOUTTOOLKIT: [
        "about_toolkit", "About &{}...".format(
        toolkit.strip('4'))
    ],
    ACTION_EXIT: ["exit", "E&xit"]
}

# action key list (needed to ensure proper ordering of actions,
# since the dictionary keys won't necessarily be ordered)
# TODO: make this dynamically update if actiondict is changed on the fly

actionkeylist = None


def update_actionkeylist():
    global actionkeylist
    actionkeylist = sorted(actiondict.keys())

update_actionkeylist()

# menu key groups (needed to properly sort and arrange menus)
menukeygroups = [
    ("&File", ACTIONS_FILEMENU),
    ("&Edit", ACTIONS_EDIT),
    ("Ac&tion", ACTIONS_ACTION),
    ("&Options", ACTIONS_OPTIONS),
    ("&Help", ACTIONS_HELP)
]


def keymenu(key):
    for title, items in menukeygroups:
        if key in items:
            return title
    return None


# button key group list (for use in adding separators)
buttonkeygroups = [ACTIONS_FILE, ACTIONS_EDIT, ACTIONS_ACTION, ACTIONS_OTHER]


# utility functions to work with key "groups"

def keygroup(key):
    for group in buttonkeygroups:
        if key in group:
            return group
    return None


def switchedgroup(lastkey, key):
    return (lastkey != 0) and (keygroup(lastkey) != keygroup(key))


# utility functions to get fully qualified pixmap path name

_pxdir = os.path.realpath(os.path.join(guipath[0], "_images"))


def pxname(aname, asuffix=None):
    if asuffix:
        return "{}-{}.png".format(aname, asuffix)
    return aname


# Return the fully qualified path name for a px file in the plib directory

def pxfile(aname):
    for asuffix in (toolkit.lower(), sys.platform, os.name, None):
        result = os.path.join(_pxdir, pxname(aname, asuffix))
        if os.path.isfile(result):
            return result
    return pxname(aname)


# Process font arguments to ensure all are filled in

def font_args(widget, font_name, font_size, bold, italic):
    if font_name is None:
        font_name = widget.get_font_name()
    if font_size is None:
        font_size = widget.get_font_size()
    if bold is None:
        bold = widget.get_font_bold()
    if italic is None:
        italic = widget.get_font_italic()
    return font_name, font_size, bold, italic


# Convenience functions to get/set elements of the actiondict

def action_caption(action):
    return actiondict[action][1]


def set_action_caption(action, caption):
    actiondict[action][1] = caption


def action_icon(action):
    return actiondict[action][0]


def action_iconfile(action):
    return pxfile(action_icon(action))


def add_action(action, icon, caption, groups=None, update=True):
    if groups is not None:
        for group, index in groups:
            if index is None:
                fn = group.append
            else:
                fn = partial(group.insert, index)
            fn(action)
    actiondict[action] = [icon, caption]
    if update:
        update_actionkeylist()


def add_actions(actions, update=True):
    for action_spec in actions:
        kw = dict(update=False)  # hack to avoid syntax error
        add_action(*action_spec, **kw)
    if update:
        update_actionkeylist()



#!/usr/bin/env python3
"""
Module EDIT -- Python Editor Objects
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Implements common 'editor' functionality for use as mixin
classes by various editing widgets. Two classes are
provided:

- PEditor: the basic editor class; keeps track of the current
  editing state and provides the appropriate hooks and
  methods to work with the object being edited.

- PFileEditor: adds file new/open/save/save as functionality.
"""

import os

from plib.gui.defs import *


# All the editor methods take no arguments, so we need a
# wrapper to take them up
def _editor_wrapper(target):
    def wrapper(*args, **kwargs):
        target()
    return wrapper


class PEditor(object):
    """Base class to provide 'editor' type methods.
    
    Can be used as a mixin or as a helper class; constructor sets up
    mainwidget field to point to appropriate widget for UI interface
    (expected to be a PMainWindow or derived class).
    
    Constructor can be passed a main widget (expected to be a PMainWindow
    or derived class so it has the appropriate methods); if not, it checks
    to see if it has a parent widget, and assumes that to be the main widget
    (for use when this is a mixin class for the client widget); otherwise it
    points to itself as the main widget (for use when this is a mixin class
    for the main widget -- note that subclasses should call this __init__
    *after* the main widget __init__).
    
    This class will work with the main widget's toolbar if it is a PToolbar,
    to keep the button enable/disable states up to date based on the current
    editing state. The 'state' includes:
    
    -- whether or not the editor is currently active ('editable')
    -- whether or not there are changes pending which have not been committed
    -- whether or not the editor has been modified but not saved
    
    The separation of 'pending' and 'modified' allows support of both
    transaction-based and file-based models in the same application.
    
    This class will update the caption of the main widget to reflect the
    'name' of the object being edited (which must be provided by overriding
    the _getname method--it does not have to be a system file name, but can
    be any string identifying the object) and the object type (which must be
    provided by overriding the _gettype method).
    """
    
    typesuffix = "Editor"
    captionformat = "{} {} - {}"
    
    signalmaps = {
        'loadsignals': 'load',
        'editsignals': 'edit',
        'changesignals': 'changed',
        'commitsignals': 'commit',
        'cancelsignals': 'cancel',
        'modifysignals': 'modify',
        'savesignals': 'save'
    }
    
    def __init__(self, mainwidget=None, control=None, data=None):
        # Figure out the main widget
        if mainwidget is not None:
            # Typically this will happen if we're a helper class
            self.mainwidget = mainwidget
        elif hasattr(self, '_parent') and (self._parent is not None):
            # Typically this will happen if we're mixed in with a
            # client widget class (e.g., in the ``scrips-edit`` example)
            self.mainwidget = self._parent
        else:
            # This should only happen if we're mixed in with a top
            # window class
            self.mainwidget = self
        if (self.mainwidget is not None):
            # This will serve for now to make sure main widget is
            # a top window instance
            if not hasattr(self.mainwidget, 'connectaction'):
                self.mainwidget = None
        
        # Figure out the control that is doing the "editing"
        if control is not None:
            # Typically this will happen if we're a helper class
            self.control = control
        elif self.mainwidget is self:
            # If we're the main widget, the editing control should be
            # our client widget (e.g., in the ``pnotepad`` example)
            self.control = self.clientwidget
        else:
            # This should only happen if we're actually mixed in with
            # the control class
            self.control = self
        
        # The data should normally be initial data to be loaded by the
        # control
        self.data = data
        
        # Flags to track state for button enable/disable
        self._editable = False
        self._pending = False
        self._modified = False
        
        # Get pointer to toolbar update method
        if self.mainwidget is not None:
            self._updatefunc = self.mainwidget.updateactions
        else:
            self._updatefunc = None
        
        # If we have a control and data, load data and populate control
        if (self.control is not None) and (self.data is not None):
            self.load()
        
        # Connect to mapped signals
        for mapname, targetname in self.signalmaps.items():
            self._setup_signals(mapname, targetname)
        
        # Connect main widget actions to our methods
        if self.mainwidget is not None:
            self._connect_mainwidget()
        
        # Connect control actions to our methods; note that this must
        # be separate from the signal mapping above because here we
        # need to preserve the signature of the control callback
        if self.control is not None:
            self._connect_control()
    
    def _setup_signals(self, mapname, targetname):
        signalmap = getattr(self, mapname, None)
        target = getattr(self, targetname, None)
        if (signalmap is not None) and (target is not None):
            for signal, attrname in signalmap.items():
                if attrname is not None:
                    source = getattr(self, attrname)
                else:
                    source = self
                
                # TODO: Get rid of this ugly hack by figuring out why the
                # wrapper doesn't get called in Qt/KDE like it should
                source.setup_notify(signal, _editor_wrapper(target))
    
    def _connect_mainwidget(self):
        self.mainwidget.connectaction(ACTION_EDIT, self.edit)
        self.mainwidget.connectaction(ACTION_COMMIT, self.commit)
        self.mainwidget.connectaction(ACTION_ROLLBACK, self.cancel)
    
    def _connect_control(self):
        """Override in derived classes to connect actions to control signals.
        """
        pass
    
    def _gettype(self):
        """Override in derived classes to customize displayed file type.
        """
        return "Object"
    
    def _getname(self):
        """Override in derived classes to customize displayed file name.
        """
        return ""
    
    def _update(self):
        if self._updatefunc is not None:
            self._updatefunc()
    
    def _get_editable(self):
        return self._editable
    
    def _set_editable(self, value):
        if self._editable != value:
            self._editable = value
            self._update()
    
    def _get_pending(self):
        return self._pending
    
    def _set_pending(self, value):
        if self._pending != value:
            self._pending = value
            self._update()
    
    def _get_modified(self):
        return self._modified
    
    def _set_modified(self, value):
        if self._modified != value:
            self._modified = value
            self._update()
    
    editable = property(_get_editable, _set_editable)
    pending = property(_get_pending, _set_pending)
    modified = property(_get_modified, _set_modified)
    
    def _donew(self):
        """Override in derived classes to create new file and populate widget.
        """
        pass
    
    def _doload(self):
        """Override in derived classes to populate widget with file data.
        """
        pass
    
    def _doedit(self):
        """Override in derived classes to activate editor.
        """
        pass
    
    def _docommit(self):
        """Override in derived classes to commit transaction.
        """
        pass
    
    def _docancel(self):
        """Override in derived classes to cancel transaction.
        """
        pass
    
    def _domodify(self):
        """Override in derived classes to modify data.
        """
        pass
    
    def _dosave(self):
        """Override in derived classes to save data from widget.
        """
        pass
    
    def _format_caption(self):
        return self.captionformat.format(
            self._gettype(), self.typesuffix, self._getname())
    
    def switchname(self):
        """Call this method to update UI for new name.
        """
        if self.mainwidget is not None:
            self.mainwidget.set_caption(self._format_caption())
    
    def new(self):
        """Call this method to open the editor with a new (unnamed) file.
        """
        self._donew()
        self.pending = False
        self.modified = False
    
    def load(self):
        """Call this method to open the editor and load data.
        """
        self._doload()
        self.pending = False
        self.modified = False
    
    def edit(self):
        """Call this method to put editor into edit mode.
        """
        if self.editable:
            self._doedit()
    
    def changed(self):
        """Call this method when a transaction starts.
        """
        if self.editable:
            self.pending = True
    
    def commit(self):
        """Call this method to commit a transaction.
        """
        self._docommit()
        self.pending = False
        self.modified = True
    
    def cancel(self):
        """Call this method to cancel a transaction.
        """
        self._docancel()
        self.pending = False
    
    def modify(self):
        """Call this method when the data is modified outside a transaction.
        """
        self._domodify()
        self.modified = True
    
    def save(self):
        """Call this method to save changes to storage.
        """
        self._dosave()
        self.modified = False
    
    def querycommit(self):
        """Ask user whether to commit uncommitted changes.
        """
        result = self.mainwidget.messagebox.query3(
            "Pending Changes",
            "There are uncommitted changes. Commit them?"
        )
        if result == answerYes:
            self.commit()
        elif result == answerNo:
            self.cancel()
        return result
    
    def querysave(self):
        """Ask user whether to save unsaved files.
        """
        result = self.mainwidget.messagebox.query3(
            "Unsaved Data",
            "There is unsaved data. Save data?"
        )
        if result == answerYes:
            self.save()
        return result
    
    def canclose(self):
        return (
            ((not self.pending) or (self.querycommit() != answerCancel))
            and ((not self.modified) or (self.querysave() != answerCancel))
        )


class PFileEditor(PEditor):
    """Editor class with richer file management capabilities.
    
    Constructor can be passed a file path (typical use expected to be as a
    helper class for a subwindow for file editing), or if not and this is
    the main widget (for use as mixin class), will open a file name dialog
    to let the user choose a file name (class constant can be changed to
    disable this behavior). The main widget's caption will show the
    pathname of the file.
    """
    
    captionformat = "{} {} [{}]"
    getfilename = True
    
    def __init__(self, mainwidget=None, filename="", control=None):
        PEditor.__init__(self, mainwidget, control)
        
        # open file
        self.filename = filename
        if self.filename:
            self._dofileopen()
        elif self.getfilename:
            self.openfile()
        else:
            self.newfile()
    
    def _connect_mainwidget(self):
        super(PFileEditor, self)._connect_mainwidget()
        # connect more main widget actions
        self.mainwidget.connectaction(ACTION_FILENEW, self.newfile)
        self.mainwidget.connectaction(ACTION_FILEOPEN, self.openfile)
        self.mainwidget.connectaction(ACTION_FILESAVE, self.savefile)
        self.mainwidget.connectaction(ACTION_FILESAVEAS, self.savefileas)
    
    def _getname(self):
        return self.filename or "<New File>"
    
    def _dofilenew(self):
        self.new()
        self.switchname()
    
    def newfile(self):
        """Create new unnamed file.
        """
        if self.canclose():
            self.filename = ""
            self._dofilenew()
    
    def _dofileopen(self):
        if self.filename:
            self.load()
            self.switchname()
    
    def openfile(self):
        """Select file name and open file.
        """
        if self.canclose():
            fname = self.mainwidget.getfiledialog()
            if len(fname) > 0:
                self.filename = str(fname)
                self._dofileopen()
    
    def _dofilesave(self):
        if self.filename:
            self.save()
            self.switchname()
    
    def savefile(self):
        """Save file to current file name, or select one if there is none.
        """
        if not self.filename:
            self.savefileas()
        else:
            self._dofilesave()
    
    def savefileas(self):
        """Select new file name and save file.
        """
        startpath, startfilter = os.path.splitext(self.filename)
        startpath = os.path.dirname(startpath)
        startfilter = "*" + startfilter
        fname = self.mainwidget.getfiledialog(startpath, startfilter,
                                              ACTION_FILESAVEAS)
        if len(fname) > 0:
            self.filename = str(fname)
            self._dofilesave()



#!/usr/bin/env python3
"""
Module KDE4DISPLAY -- Python KDE Text Display Widgets
Sub-Package GUI.TOOLKITS.KDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the KDE 4 GUI objects for text display widgets.
"""

from PyQt4 import Qt as qt
from PyKDE4 import kdeui

from plib.gui._widgets import display

from ._kde4common import _PKDEWidget


class PKDETextDisplay(_PKDEWidget, kdeui.KTextEdit,
                      display.PTextDisplayBase):
    
    widget_class = kdeui.KTextEdit
    
    def __init__(self, parent, text=None, geometry=None, scrolling=False):
        kdeui.KTextEdit.__init__(self, parent)
        # KDE forces background to gray on readonly as well as disabled,
        # fixup here
        palette = qt.QPalette(self.palette())
        palette.setColor(self.backgroundRole(),
                         palette.color(qt.QPalette.Active, qt.QPalette.Base))
        self.setPalette(palette)
        self.setReadOnly(True)
        if scrolling:
            self.setLineWrapMode(qt.QTextEdit.NoWrap)
        display.PTextDisplayBase.__init__(self, text, geometry)
    
    def get_text(self):
        return str(self.toPlainText())
    
    def set_text(self, value):
        self.setPlainText(value)
    
    def set_width(self, width):
        self.resize(width, self.height())
        # KDE text edits don't seem to fully respect
        # qt.QSizePolicy.MinimumExpanding
        self.setMinimumWidth(width)
    
    def set_height(self, height):
        self.resize(self.width(), height)
        # KDE text edits don't seem to fully respect
        # qt.QSizePolicy.MinimumExpanding
        self.setMinimumHeight(height)



#!/usr/bin/env python3
"""
Module WXSTATUSBAR -- Python wxWidgets Status Bar Objects
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI objects for the status bar.
"""

import wx

from plib.gui._base import statusbar

from ._wxcommon import _PWxWidgetBase
#from _wxlabel import PWxTextLabel


class PWxStatusBar(_PWxWidgetBase, wx.StatusBar, statusbar.PStatusBarBase):
    
    #textareaclass = PWxTextLabel # FIXME: not needed for wx?
    
    def __init__(self, parent, widgets=None):
        wx.StatusBar.__init__(self, parent)
        self.SetFieldsCount(1)  # FIXME: add fields for 'widgets' below
        self.SetStatusWidths([-1])
        statusbar.PStatusBarBase.__init__(self, parent, widgets)
    
    def _add_widget(self, widget, expand=False, custom=True):
        # FIXME: No custom widgets in wx?
        pass
    
    def get_text(self):
        return self.GetStatusText(0)
    
    def set_text(self, value):
        self.SetStatusText(value, 0)



#!/usr/bin/env python3
"""
PNOTEPAD.PY
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

A simple plain text editor using the PLIB3 package.
"""

import sys
import os

from plib.gui import __version__
from plib.gui import main as gui
from plib.gui import common
from plib.gui.defs import *


class PNotepad(gui.PMainWindow, gui.PTextFileEditor):
    
    aboutdata = {
        'name': "PNotepad",
        'version': __version__,
        'copyright': "Copyright (C) 2008-2013 by Peter A. Donis",
        'license': "GNU General Public License (GPL) Version 2",
        'description': "Plain Text Editor",
        'developers': ["Peter Donis"],
        'website': "http://www.peterdonis.net",
        'icon': os.path.join(os.path.split(os.path.realpath(__file__))[0],
                             "pnotepad.png")
    }
    
    # FIXME: Add prefsdata to illustrate combined API with ini file
    
    actionflags = [
        ACTION_FILENEW, ACTION_FILEOPEN, ACTION_FILESAVE, ACTION_FILESAVEAS,
        ACTION_EDITUNDO, ACTION_EDITREDO,
        ACTION_EDITCUT, ACTION_EDITCOPY, ACTION_EDITPASTE,
        ACTION_EDITDELETE, ACTION_EDITSELECTALL,
        ACTION_ABOUT, ACTION_ABOUTTOOLKIT, ACTION_EXIT
    ]
    
    clientwidgetclass = gui.PEditControl
    defaultcaption = "Plain Text Editor"
    placement = (SIZE_MAXIMIZED, MOVE_NONE)
    queryonexit = False
    
    def __init__(self, app):
        gui.PMainWindow.__init__(self, app)
        self.clientwidget.set_background_color(COLOR_WHITE)
        self.clientwidget.set_font("Courier New", 12)
        if len(sys.argv) > 1:
            fname = sys.argv[1]
        else:
            fname = ""
        gui.PTextFileEditor.__init__(self, filename=fname)
        self.editable = True
        
        # Check to make sure everything initialized properly
        assert self.editor is self
        assert self.mainwidget is self
        assert self.control is self.clientwidget
    
    def _gettype(self):
        return "Plain Text"
    
    def _donew(self):
        gui.PTextFileEditor._donew(self)
        self.statusbar.set_text("Editor cleared.")
    
    def _doload(self):
        gui.PTextFileEditor._doload(self)
        self.statusbar.set_text("File data read.")
    
    def _dosave(self):
        gui.PTextFileEditor._dosave(self)
        self.statusbar.set_text("File data written.")


if __name__ == "__main__":
    gui.runapp(PNotepad)



#!/usr/bin/env python3
"""
Internal Module _GUI
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the code to determine what GUI toolkit
should be used. It is for internal use by the other GUI
modules; user applications should not need to import it
since the GUI sub-package itself contains the gui_toolkit
variable in its namespace.
"""

import sys
import os

from plib.gui.defs import GUI_WX, GUI_KDE4, GUI_QT4, GUI_PYSIDE
from plib.gui._setup import *

# This variable is provided so that if you want to test your own toolkit
# that's not in the supported list, you can still use the other facilities;
# you just need to import this module and set this variable to True before
# importing plib.gui.main. However, note that if you do this, *none* of the
# standard GUI widget classes will be available unless you add them yourself;
# you can do this by setting attributes on this module before importing
# plib.gui.main, or by constructing all your widgets "by hand". The former
# technique has the advantage of allowing you to use PLIB3.GUI's other
# built-in facilities, but it requires you to provide classes with all the
# appropriate names yourself (they can be dummy classes if you won't use
# them, but they must be there). For example:
#
# from plib.gui import _gui
# _gui.gui_test = True
# _gui.PApplication = PMyApplication # assuming this was defined earlier
# _gui.PTopWindow = PMyTopWindow
# ... etc. for all other GUI widget classes
#
# Once you do this, you can use the rest of PLIB3.GUI normally, including
# classes like PAutoPanel that allow you to define GUI specs declaratively.
gui_test = False

# GUI toolkit determination -- this has to be here to avoid circular imports
# among the various GUI modules
gui_toolkit = 0

# If environment variable present and correct, it determines the toolkit
env_toolkit_name = os.getenv("GUI_TOOLKIT")
if env_toolkit_name is not None:
    # Hack to allow env var to be either numeric value or name of the constant
    try:
        gui_toolkit = int(env_toolkit_name)
    except ValueError:
        for name in dir(sys.modules[__name__]):
            if name == env_toolkit_name:
                gui_toolkit = getattr(sys.modules[__name__], name)
                break
        del name

# Clean up namespace
del env_toolkit_name

# If toolkit not set above, detect desktop environment
if gui_toolkit == 0:
    desktop_session = os.getenv('DESKTOP_SESSION')
    window_mgr = os.getenv('WINDOWMANAGER')
    
    # check KDE/Qt first
    if KDE4_PRESENT or QT4_PRESENT or PYSIDE_PRESENT:
        kde_session = 'KDE_FULL_SESSION' in os.environ
        kde_desktop = (desktop_session in ('kde', 'trinity'))
        kde_winmgr = isinstance(window_mgr, str) and \
            (window_mgr.find('kde') > -1)
        # TODO: check if kdeinit (kde root process) and kwin (kde
        # window mgr) are running
        
        # if KDE is running, default to KDE if PyKDE is present,
        # otherwise Qt; but make sure the versions match
        if kde_session or kde_desktop or kde_winmgr:
            if os.getenv('KDE_SESSION_VERSION') == '4':
                if KDE4_PRESENT:
                    gui_toolkit = GUI_KDE4
                elif QT4_PRESENT:
                    gui_toolkit = GUI_QT4
                elif PYSIDE_PRESENT:
                    # This is the last choice because it appears to be less
                    # "canonical" than PyQt4
                    gui_toolkit = GUI_PYSIDE
        
        # clean up namespace
        del kde_session, kde_desktop, kde_winmgr
    
    # check Qt 4 if none of the above worked
    if (gui_toolkit == 0) and QT4_PRESENT:
        gui_toolkit = GUI_QT4
    
    # check PySide if none of the above worked
    if (gui_toolkit == 0) and PYSIDE_PRESENT:
        gui_toolkit = GUI_PYSIDE
    
    # default to wxWidgets if present
    if (gui_toolkit == 0) and WX_PRESENT:
        gui_toolkit = GUI_WX
    
    # clean up namespace
    del desktop_session, window_mgr

del sys, os



#!/usr/bin/env python3
"""
Module PYSIDEDISPLAY -- Python PySide Text Display Widgets
Sub-Package GUI.TOOLKITS.PYSIDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PySide GUI objects for text display widgets.
"""

from PySide import QtGui as qt

from plib.gui._widgets import display

from ._pysidecommon import _PQtWidgetMeta, _PQtWidget


class PQtTextDisplay(_PQtWidget, qt.QTextEdit, display.PTextDisplayBase, metaclass=_PQtWidgetMeta):
    
    def __init__(self, parent, text=None, geometry=None, scrolling=False):
        qt.QTextEdit.__init__(self, parent)
        self.setReadOnly(True)
        if scrolling:
            self.setLineWrapMode(qt.QTextEdit.NoWrap)
        display.PTextDisplayBase.__init__(self, text, geometry)
    
    def get_text(self):
        return str(self.toPlainText())
    
    def set_text(self, value):
        self.setPlainText(value)
    
    def set_width(self, width):
        self.resize(width, self.height())
        # Qt text edits don't seem to fully respect
        # qt.QSizePolicy.MinimumExpanding
        self.setMinimumWidth(width)
    
    def set_height(self, height):
        self.resize(self.width(), height)
        # Qt text edits don't seem to fully respect
        # qt.QSizePolicy.MinimumExpanding
        self.setMinimumHeight(height)



#!/usr/bin/env python3
"""
Module WXCOMBO -- Python wxWidgets Combo Box Widgets
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI objects for combo boxes.
"""

import wx

from plib.gui.defs import *
from plib.gui._widgets import combo

from ._wxcommon import _PWxWidget


# TODO: determine if simple combo without edit cell can be used
class PWxComboBox(_PWxWidget, wx.ComboBox, combo.PComboBoxBase):
    
    def __init__(self, parent, sequence=None, target=None, geometry=None):
        wx.ComboBox.__init__(self, parent,
                             style=(wx.CB_DROPDOWN | wx.CB_READONLY))
        self._align = ALIGN_LEFT  # used by PWxPanel to determine placement
        combo.PComboBoxBase.__init__(self, sequence, target, geometry)
    
    def current_text(self):
        return self.GetStringSelection()
    
    def set_current_text(self, text):
        self.SetStringSelection(text)
    
    def current_index(self):
        return self.GetSelection()
    
    def set_current_index(self, index):
        self.SetSelection(index)
    
    def _indexlen(self):
        return self.GetCount()
    
    def _get_data(self, index):
        return self.GetString(index)
    
    def _set_data(self, index, value):
        self.SetString(index, value)
    
    def _add_data(self, index, value):
        if index == self.__len__():
            self.Append(value)
        else:
            self.Insert(value, index)
    
    def _del_data(self, index):
        self.Delete(index)



#!/usr/bin/env python3
"""
Sub-Package TEST of Package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This is a namespace package for the PLIB3 test suite.
"""

from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)



#!/usr/bin/env python3
"""
Module SPECS -- GUI Specification Helper Module
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains helper variables and functions for
defining GUI specifications that can be used by the PLIB3
"auto" GUI classes. For an example of usage, see the
pyidserver-gui.py example program.
"""

from plib.gui import main as gui
from plib.gui.common import actiondict, AUTO_TARGET

# Control variables for widget geometries; if you want to
# change these values, you can import specs and mutate them
# as long as you do it before any widgets are constructed

framemargin = 10
tabmargin = 10
panelmargin = 4
panelspacing = 10
buttonwidth = 110
combowidth = 90
editwidth = 210
numeditwidth = 80
labelwidth = 150
listwidth = 250
mainwidth = 400
mainheight = 250

# Classes to be used for each basic widget type; these can also be
# changed to point to custom classes as long as it's done before
# widget creation; note that any such changes will be *global*
# (i.e., they affect all modules using this one in your application),
# so if you just want custom widgets for a particular GUI window or
# dialog, it's better to build their specs by hand (see the _dialogs
# module for an example of how this can be done, with PPrefsDialog)

# These are functions so that the loading of the actual widget modules
# gets deferred until the specs are realized (instead of happening
# when this module is imported), since plib.gui.main itself does not
# load widget classes until the module attributes are accessed

button_class = lambda: gui.PButton
checkbox_class = lambda: gui.PCheckBox
combo_class = lambda: gui.PComboBox
sorted_combo_class = lambda: gui.PSortedComboBox
edit_class = lambda: gui.PEditBox
label_class = lambda: gui.PTextLabel
listbox_class = lambda: gui.PListBox
sorted_listbox_class = lambda: gui.PSortedListBox
listview_class = lambda: gui.PListView
sorted_listview_class = lambda: gui.PSortedListView
memo_class = lambda: gui.PEditControl
table_class = lambda: gui.PTable
text_class = lambda: gui.PTextDisplay

groupbox_class = lambda: gui.PAutoGroupBox
tabwidget_class = lambda: gui.PAutoTabWidget

# Note that we don't need to define a panel class here because any class
# derived from PAutoPanel will automatically use itself to instantiate
# sub-panels; thus all the panel specs here are just lists of sub-widget
# specs for use by whatever panel class you choose.

# These are functions rather than static tuples so that the values
# get pulled dynamically from the above variables

button_geometry = lambda: (None, None, buttonwidth, None)
combo_geometry = lambda: (None, None, combowidth, None)
editgeometry = lambda: (None, None, editwidth, None)
numeditgeometry = lambda: (None, None, numeditwidth, None)
listboxgeometry = lambda: (None, None, listwidth, None)
labelgeometry = lambda: (None, None, labelwidth, None)
main_geometry = lambda: (None, None, mainwidth, mainheight)

# Convenience class to provide header labels with the appropriate
# attributes for the table/list widget APIs; we don't bother deferring
# the loading of the class here since it's just a small one

header_class = gui.PHeaderLabel


# Convenience function for enabling auto-target functionality

def _adjust_target(target, fstr, name):
    # Note that "no target" is None, which isn't touched here
    if target in ('', AUTO_TARGET):
        newtarget = fstr.format(name)
        if target == '':
            return newtarget
        return (target, newtarget)
    return target


# Convenience functions for building specs


def get_padding(name):
    return ([], (), {}, 'padding_{}'.format(name))


def _label_geometry(width):
    if width:
        return (None, None, width, None)
    return labelgeometry()


def get_label(label, name, width=None):
    return (
        label_class,
        ("{}: ".format(label),),
        {'geometry': _label_geometry(width)},
        'label_{}'.format(name)
    )


def get_textlabel(label, name, width=None):
    return (
        label_class,
        (label,),
        {'geometry': _label_geometry(width)},
        'label_{}'.format(name)
    )


def get_checkbox(label, name, target=None):
    return (
        checkbox_class,
        (label,),
        {'target': _adjust_target(target, '{}_toggled', name)},
        'checkbox_{}'.format(name)
    )


def get_combobox(items, name, target=None):
    return (
        combo_class,
        (items,),
        {'target': _adjust_target(target, '{}_selected', name),
            'geometry': combo_geometry()},
        'combo_{}'.format(name)
    )


def get_sorted_combobox(items, name, target=None, key=None):
    return (
        sorted_combo_class,
        (items,),
        {'target': _adjust_target(target, '{}_selected', name),
            'geometry': combo_geometry(), 'key': key},
        'combo_{}'.format(name)
    )


def get_editbox(name, target=None, expand=True):
    return (
        edit_class,
        (),
        {'target': _adjust_target(target, '{}_changed', name),
            'geometry': editgeometry(), 'expand': expand},
        'edit_{}'.format(name)
    )


def get_numeditbox(name, target=None, expand=True):
    return (
        edit_class,
        (),
        {'target': _adjust_target(target, '{}_changed', name),
            'geometry': numeditgeometry(), 'expand': expand},
        'edit_{}'.format(name)
    )


def get_listbox(labels, name, target=None):
    # FIXME: this doesn't appear to properly handle labels with width = -1
    return (
        listbox_class,
        ([header_class(*args) for args in labels],),
        {'target': _adjust_target(target, '{}_selected', name),
            'geometry': listboxgeometry()},
        'listbox_{}'.format(name)
    )


def get_sorted_listbox(labels, name, target=None, key=None):
    # FIXME: this doesn't appear to properly handle labels with width = -1
    return (
        sorted_listbox_class,
        ([header_class(*args) for args in labels],),
        {'target': _adjust_target(target, '{}_selected', name),
            'geometry': listboxgeometry(), 'key': key},
        'listbox_{}'.format(name)
    )


def get_textcontrol(name, text="", scrolling=True):
    return (
        text_class,
        (text,),
        {'geometry': main_geometry(), 'scrolling': scrolling},
        'text_{}'.format(name)
    )


def get_textdisplay(name, text="", scrolling=True):
    return (
        text_class,
        (text,),
        {'scrolling': scrolling},
        'text_{}'.format(name)
    )


def get_editcontrol(name, target=None, scrolling=True):
    return (
        memo_class,
        (),
        {'target': _adjust_target(target, '{}_changed', name),
            'geometry': main_geometry(), 'scrolling': scrolling},
        'memo_{}'.format(name)
    )


def get_memo(name, target=None, scrolling=True):
    return (
        memo_class,
        (),
        {'target': _adjust_target(target, '{}_changed', name),
            'scrolling': scrolling},
        'memo_{}'.format(name)
    )


def get_listview(labels, name, target=None):
    # FIXME: this doesn't appear to properly handle labels with width = -1
    return (
        listview_class,
        ([header_class(*args) for args in labels],),
        {'target': _adjust_target(target, '{}_selected', name)},
        'listview_{}'.format(name)
    )


def get_sorted_listview(labels, name, target=None, key=None):
    # FIXME: this doesn't appear to properly handle labels with width = -1
    return (
        sorted_listview_class,
        ([header_class(*args) for args in labels],),
        {'target': _adjust_target(target, '{}_selected', name), 'key': key},
        'listview_{}'.format(name)
    )


def get_table(labels, name, target=None):
    return (
        table_class,
        ([header_class(*args) for args in labels],),
        {'target': _adjust_target(target, '{}_changed', name)},
        'table_{}'.format(name)
    )


def get_button(caption, name, pxname=None, target=''):
    return (
        button_class,
        (caption, pxname),
        {'target': _adjust_target(target, '{}', name),
            'geometry': button_geometry()},
        'button_{}'.format(name)
    )


def get_action_button(action, name=None, target=''):
    # TODO: do we need more sophisticated error handling here?
    if name is None:
        try:
            name = actiondict[action][0]
        except KeyError:
            name = "Unknown"
    if target == '':
        if name != "Unknown":
            target = name
        else:
            target = None
    return (
        button_class,
        (action, ""),
        {'target': _adjust_target(target, '{}', name),
            'geometry': button_geometry()},
        'button_{}'.format(name)
    )


def get_interior_panel(contents, align, layout, name):
    return (
        contents,
        (),
        {'align': align, 'layout': layout},
        'panel_{}'.format(name)
    )


def get_midlevel_panel(contents, align, layout, name):
    return (
        contents,
        (),
        {'align': align, 'layout': layout,
            'spacing': panelspacing},
        'panel_{}'.format(name)
    )


def get_groupbox(contents, caption, name):
    return (
        groupbox_class,
        (caption, contents),
        {'margin': tabmargin, 'spacing': panelspacing},
        'groupbox_{}'.format(name)
    )


def get_tab_panel(contents, align, layout, name):
    return (
        contents,
        (),
        {'align': align, 'layout': layout,
            'margin': tabmargin, 'spacing': panelspacing},
        'panel_{}'.format(name)
    )


def get_tabwidget(tabs, name, target=None):
    return (
        tabwidget_class,
        (tabs,),
        {'target': _adjust_target(target, '{}_changed', name)},
        'tabs_{}'.format(name)
    )


def get_toplevel_panel(contents, align, layout, name):
    return (
        contents,
        (),
        {'align': align, 'layout': layout,
            'margin': panelmargin, 'spacing': panelspacing},
        'panel_{}'.format(name)
    )


### Alternate API (this will become the norm in version 0.8)

def toplevel_panel(align, layout, name, contents):
    return get_toplevel_panel(contents, align, layout, name)


def groupbox(caption, name, contents):
    return get_groupbox(contents, caption, name)


def tabwidget(name, tabs, target=None):
    return get_tabwidget(tabs, name, target)


def midlevel_panel(align, layout, name, contents):
    return get_midlevel_panel(contents, align, layout, name)


def tab_panel(align, layout, name, contents):
    return get_tab_panel(contents, align, layout, name)


def interior_panel(align, layout, name, contents):
    return get_interior_panel(contents, align, layout, name)


# New API methods, use "auto-targeting" by default

def checkbox(label, name, target=AUTO_TARGET):
    return get_checkbox(label, name, target)


def combobox(name, items, target=AUTO_TARGET):
    return get_combobox(items, name, target)


def sorted_combobox(name, items, target=AUTO_TARGET, key=None):
    return get_sorted_combobox(items, name, target, key)


def editbox(name, target=AUTO_TARGET, expand=True):
    return get_editbox(name, target, expand)


def editcontrol(name, target=AUTO_TARGET, scrolling=True):
    return get_editcontrol(name, target, scrolling)


def listbox(name, labels, target=AUTO_TARGET):
    return get_listbox(labels, name, target)


def sorted_listbox(name, labels, target=AUTO_TARGET, key=None):
    return get_sorted_listbox(labels, name, target, key)


def listview(name, labels, target=AUTO_TARGET):
    return get_listview(labels, name, target)


def sorted_listview(name, labels, target=AUTO_TARGET, key=None):
    return get_sorted_listview(labels, name, target, key)


def table(name, labels, target=AUTO_TARGET):
    return get_table(labels, name, target)


def memo(name, target=AUTO_TARGET, scrolling=True):
    return get_memo(name, target, scrolling)


def numeditbox(name, target=AUTO_TARGET, expand=True):
    return get_numeditbox(name, target, expand)


# New API methods for controls that don't need or don't
# use auto-targeting (the latter are the buttons, which
# should always complain if they don't find event handlers
# since that's the only point of having them)

action_button = get_action_button
button = get_button
label = get_label
padding = get_padding
textcontrol = get_textcontrol
textdisplay = get_textdisplay
textlabel = get_textlabel



#!/usr/bin/env python3
"""
Module QT4NOTIFY -- Python Qt Common Objects
Sub-Package GUI.TOOLKITS.QT4 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the Qt 4 GUI socket notifier objects.
"""

from PyQt4 import Qt as qt

from plib.gui.defs import *

from ._qt4common import _PQtCommunicator

_qtnotifytypes = {
    NOTIFY_READ: qt.QSocketNotifier.Read,
    NOTIFY_WRITE: qt.QSocketNotifier.Write
}


class PQtSocketNotifier(_PQtCommunicator, qt.QSocketNotifier):
    
    auto_enable = True
    
    def __init__(self, obj, notify_type, select_fn, notify_fn):
        self._obj = obj
        self.select_fn = select_fn
        self.notify_fn = notify_fn
        qt.QSocketNotifier.__init__(self, obj.fileno(),
                                    _qtnotifytypes[notify_type])
        self.setup_notify(SIGNAL_NOTIFIER, self.handle_notify)
    
    def set_enabled(self, enable):
        self.setEnabled(enable)
    
    def handle_notify(self, sock):
        self.set_enabled(False)
        if (sock == self._obj.fileno()) and self.select_fn():
            self.notify_fn(self._obj)
        if self.auto_enable and self.select_fn():
            self.set_enabled(True)



#!/usr/bin/env python3
"""
Module MIXINS -- Mixin Classes for GUI
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines mixin classes to customize behavior of GUI objects.
The following classes are included:

- PTableMixin: combines PEditor with table-type widget.

- PTreeMixin: combines PEditor with tree-type widget.

- PTextMixin: combines PEditor or PFileEditor with text-editing
  widget.

- PPanelMixin: combines panel widget with automated generation
  of sub-widgets based on a list of widget specs (each spec
  is a 4-tuple as described in the class docstring). The specs
  can be generated by hand, but it is usually easier to use
  the pre-built convenience functions in the ``plib.gui.specs``
  module.

- PTabMixin, PGroupMixin: give tab and groupbox widgets the
  ability to auto-construct sub-widgets based on specs.

- PStatusMixin: allows status bar widget to auto-construct its
  child widgets based on specs.
"""

import types

from plib.gui.defs import *
from plib.gui import main


# Mixin classes to customize editor behavior for various types of widgets;
# note that these should appear *before* the editor class in the list of
# base classes.

class PTableMixin(object):
    """ Mixin class for editor with table widget. """
    
    _data_loaded = False
    
    def cellselected(self, row, col):
        """Call event handler for cell selection.
        """
        self._on_cellselected(row, col)
    
    def _on_cellselected(self, row, col):
        """Hook to take other actions when cell is selected.
        """
        pass
    
    def tablechanged(self, row, col):
        """Update data store with changed value from table cell.
        """
        if not self.control[row]._updating:
            self.data[row][col] = self.control[row][col]
            self.modified = True
            self._on_tablechanged(row, col)
    
    def _on_tablechanged(self, row, col):
        """Hook to take other actions when table is changed.
        """
        pass
        
        # TODO: Add this to PTableEditor?
    
    def _helperadd(self, index, value):
        """Update data store with new row's value
         
        Note: only update if data is already loaded.
        """
        
        if self._data_loaded:
            self.data.insert(index, value)
        super(PTableMixin, self)._helperadd(index, value)
    
    def _helperdel(self, index, item):
        """Remove deleted item from data store.
        """
        super(PTableMixin, self)._helperdel(index, item)
        if self._data_loaded:
            del self.data[index]
    
    def _connect_control(self):
        """Connect to table changed signal.
        """
        self.control.setup_notify(SIGNAL_CELLSELECTED, self.cellselected)
        self.control.setup_notify(SIGNAL_TABLECHANGED, self.tablechanged)
    
    def _doload(self):
        """Populate table widget from data.
        
        Assumes that data is a sequence of sequences; each top-level
        sequence item is a table row, each item within the row is data
        for a cell.
        """
        
        self.control.extend(self.data)
        # Since the widget wasn't initialized with data, we have to size
        # it here
        self.control.set_min_size(
            self.control.minwidth(), self.control.minheight())
        # Update the loaded field so further row adds/deletes will update
        # the data store
        self._data_loaded = True


class PTreeMixin(object):
    """Mixin class for editor with tree-type widget.
    """
    
    def _on_treeselected(self):
        """Override to take action on new item selection in tree.
        """
        pass
    
    def _connect_control(self):
        """Connect to list view changed signal.
        """
        self.control.setup_notify(SIGNAL_LISTSELECTED, self._on_treeselected)
    
    def _doload(self):
        """Populate tree widget from data.
        
        Assume that data is a sequence of 2-tuples, with the first item being
        a sequence of item strings (to allow for multiple columns), and the
        second item being a sub-sequence of 2-tuples for child tree nodes.
        """
        self.control.extend(self.data)


class PTextMixin(object):
    """Mixin class for editor with text control widget.
    """
    
    modifysignals = {SIGNAL_TEXTCHANGED: 'control'}
    
    def _gettype(self):
        return "Text"
    
    def _connect_mainwidget(self):
        super(PTextMixin, self)._connect_mainwidget()
        # connect editing actions
        self.mainwidget.connectaction(ACTION_EDITUNDO, self.editundo)
        self.mainwidget.connectaction(ACTION_EDITREDO, self.editredo)
        self.mainwidget.connectaction(ACTION_EDITCUT, self.editcut)
        self.mainwidget.connectaction(ACTION_EDITCOPY, self.editcopy)
        self.mainwidget.connectaction(ACTION_EDITPASTE, self.editpaste)
        self.mainwidget.connectaction(ACTION_EDITDELETE, self.editdelete)
        self.mainwidget.connectaction(ACTION_EDITSELECTALL, self.selectall)
    
    def _connect_control(self):
        """Connect to edit control state changed signal.
        """
        self.control.setup_notify(SIGNAL_TEXTSTATECHANGED,
                                  self._on_statechanged)
        self.statelist = [
            ([ACTION_EDITUNDO], self.control.can_undo),
            ([ACTION_EDITREDO], self.control.can_redo),
            ([ACTION_EDITPASTE], self.control.can_paste),
            (
                [ACTION_EDITCUT, ACTION_EDITCOPY, ACTION_EDITDELETE],
                self.control.can_clip
            )
        ]
        self._on_statechanged()
    
    def _on_statechanged(self):
        for keylist, flagfunc in self.statelist:
            for key in keylist:
                self.mainwidget._update_action(key, flagfunc())
    
    def editundo(self):
        self.control.undo_last()
    
    def editredo(self):
        self.control.redo_last()
    
    def editcut(self):
        self.control.cut_to_clipboard()
    
    def editcopy(self):
        self.control.copy_to_clipboard()
    
    def editpaste(self):
        self.control.paste_from_clipboard()
    
    def editdelete(self):
        self.control.delete_selected()
    
    def selectall(self):
        self.control.select_all()


class PTextFileMixin(PTextMixin):
    """Mixin class for file editor with text control widget.
    """
    
    def _gettype(self):
        return "Text File"
    
    def _donew(self):
        self.control.clear_edit()
    
    def _filedata_to_control(self, filedata):
        self.control.edit_text = filedata
    
    def _doload(self):
        filedata = ""
        f = open(self.filename, 'rU')
        try:
            filedata = f.read()
        finally:
            f.close()
        if filedata:
            self._filedata_to_control(filedata)
    
    def _filedata_from_control(self):
        return self.control.edit_text
    
    def _dosave(self):
        filedata = self._filedata_from_control()
        if filedata:
            f = open(self.filename, 'w')
            try:
                f.write(filedata)
            finally:
                f.close()


# Mixin classes to enable automatic GUI generation from specifications
# stored in Python lists, tuples, and dicts

class _PAutoBase(object):
    """Base class for GUI auto-generation.
    
    Base class for mixins that allow automatic construction of
    sub-widgets from specs.
    """
    
    def _widget_from_klass(self, klass, args, kwds, attrname):
        # Hack to allow patching in attribute name if needed
        if hasattr(klass, '_use_attrname') and klass._use_attrname:
            return klass(self, attrname, *args, **kwds)
        return klass(self, *args, **kwds)
    
    def widget_from_spec(self, spec, fn=None):
        """Construct widget from spec.
        
        Constructs widget from spec, putting self as first constructor
        argument (i.e., assumes that widget constructor takes 'parent' as
        first arg), and replacing ``target`` keyword argument, if present,
        with the actual callable (found by looking for a method on self,
        or in self's parent tree, with the given name).
        """
        
        klass, args, kwds, attrname = spec
        if 'target' in kwds:
            targetspec = kwds['target']
            if isinstance(targetspec, tuple):
                auto, targetname = targetspec
                assert auto == AUTO_TARGET  # specs should guarantee this
            else:
                targetname = targetspec
                auto = False
            if isinstance(targetname, str) and targetname:
                # Look up the method by name, starting with self and
                # walking the parent tree
                p = self
                target = None
                # FIXME: wtf do we need p is not None below? For some crazy
                # reason, a PAutoTabWidget instance registers as boolean False???
                while (p is not None) and not target:
                    try:
                        target = getattr(p, targetname)
                    except AttributeError:
                        p = getattr(p, '_parent', None)
                if not (target or auto):
                    raise ValueError(
                        "Could not find target method {} for widget {}".format(
                        targetname, attrname)
                    )
                kwds['target'] = target
        # This allows us to defer loading the actual module containing
        # the widget class until here (instead of forcing it to happen
        # when the specs module is imported)
        if isinstance(klass, types.FunctionType):
            klass = klass()
        result = self._widget_from_klass(klass, args, kwds, attrname)
        if fn is not None:
            fn(result, attrname)
        elif attrname is not None:
            setattr(self, attrname, result)
        return result


class PPanelMixin(_PAutoBase):
    """Mixin class for panel that auto-generates child widgets.
    
    Mixin class that provides an easier way to create child
    widgets for panels. The childlist class field is used
    to provide a list of 4-tuples (class, args, kwds, attrname)
    which is used to guide the creation of child widgets. The
    attrname field, if not None, tells what named attribute of
    the panel the created widget should be bound to.
    
    Note that this method requires that, if any of the child
    widgets are themselves panels, they will require their own
    classes to fill in their childlist fields. However, an
    alternative is provided: if the baseclass class field is
    filled in, then the 'class' member of the 4-tuples in
    childlist may instead be a sub-list of 4-tuples; the
    sub-list will then be used to create a panel class on
    the fly, subclassed from baseclass, and with the given
    list as its childlist. (As long as baseclass is derived
    from PPanelMixin, this process will work recursively.)
    
    As an added feature, this class overrides __getattr__ to
    allow attributes of child panels to appear as attributes
    of the parent panel. This is mainly to allow easier access
    to controls defined in child panels (so you don't have to
    remember at which level of nested panels you inserted a
    control -- just treat it as part of the main panel).
    """
    
    baseclass = None
    childlist = []
    panelclass = None
    
    def _widget_from_klass(self, klass, args, kwds, attrname):
        if not isinstance(klass, type):
            # klass is a sub-list, create a class on the fly
            klass = type(self.baseclass)(
                "{}_{}".format(self.__class__.__name__, attrname),
                (self.baseclass,),
                {'baseclass': self.baseclass, 'childlist': klass})
        return _PAutoBase._widget_from_klass(
            self, klass, args, kwds, attrname)
    
    def _createpanels(self):
        """Create child widgets listed in class field.
        """
        for spec in self.childlist:
            widget = self.widget_from_spec(spec)
            # Panel-type widgets add themselves, others we need to add here
            if not isinstance(widget, self.panelclass):
                self._addwidget(widget)
    
    def __getattr__(self, name):
        """Allow 'pass-through' access to attributes of child panels.
        """
        try:
            # This is in case we're mixing in with a class that also
            # overrides __getattr__
            result = super(PPanelMixin, self).__getattr__(name)
            return result
        except AttributeError:
            # Now loop through child panels
            for panel in self.__dict__['_panels']:
                try:
                    result = getattr(panel, name)
                    return result
                except AttributeError:
                    pass
        # If we get here, nothing was found
        raise AttributeError("{} object has no attribute '{}'".format(
                             self.__class__.__name__, name))


class _PAutoWithPanels(_PAutoBase):
    """Base class for auto-constructing widgets with parent panels.
    
    An auto-constructing class that groks having a ``PAutoPanel`` as a parent
    (it allows the parent panel to 'pass through' attribute access to this
    object's children) and having ``PAutoPanel`` specs in its spec list.
    """
    
    basename = 'with_panels'
    panelclass = None
    
    def _widget_from_klass(self, klass, args, kwds, attrname):
        if not isinstance(klass, type):
            # klass is panel contents, wrap panel around it.
            base = self.panelclass
            if isinstance(self._parent, self.panelclass):
                basename = self._parent.__class__.__name__
            else:
                basename = self.basename
            klass = type(base)("{}_{}".format(basename, attrname),
                               (base,), {'childlist': klass})
        return _PAutoBase._widget_from_klass(
            self, klass, args, kwds, attrname)
    
    def _process_control(self, control, attrname):
        if isinstance(self._parent, self.panelclass):
            if attrname is not None:
                # 'Pass-through' this control
                setattr(self._parent, attrname, control)
            if isinstance(control, self.panelclass):
                # So finding controls by attr name works
                self._parent._panels.append(control)
    
    def widget_from_spec(self, spec):
        return _PAutoBase.widget_from_spec(self, spec, self._process_control)


class PTabMixin(_PAutoWithPanels):
    """Mixin class for auto-generating tab widgets.
    
    Mixin class for tab widgets that 'expands' a list of panel specs
    into the actual panels and adds them as tabs.
    """
    
    basename = 'tab'
    
    def _createtabs(self, tabs):
        """Create tabs from specs.
        
        Assumes that tabs is a sequence of 2-tuples containing
        (title, panelspec) for each tab.
        """
        
        # FIXME: The extend method doesn't take an iterator? (if it did,
        # we could just pass a generator expression to extend here instead
        # of the for loop with append, but that doesn't seem to work)
        for title, spec in tabs:
            self.append((title, self.widget_from_spec(spec)))


class PGroupMixin(_PAutoWithPanels):
    """Mixin class for auto-generating group boxes.
    
    Mixin class to auto-construct group box controls based
    on specs list.
    """
    
    basename = 'group'
    
    def _process_control(self, control, attrname):
        _PAutoWithPanels._process_control(self, control, attrname)
        self._add_control(control)
        self._controls[attrname] = control
    
    def _init_controls(self, controls):
        """ Assume controls is a sequence of specs. """
        if self._controls is None:
            self._controls = {}
        for control in controls:
            self.widget_from_spec(control)


def _widget_helper(wclassname, args, kwds, wname):
    def f():
        w = getattr(main, wclassname)(*args, **kwds)
        return (wname, w)
    f.__name__ = '{}_{}'.format(wclassname, wname)
    return f


class PStatusMixin(object):
    """Mixin class for auto-generating status bar.
    
    Mixin class to allow easier specification of status bar widgets.
    The widgetspecs class field should be filled in with a list of
    4-tuples similar to those used by PPanelMixin above; the tuples
    should contain (classname, args, kwds, attrname), where classname
    is the widget class name, which is assumed to be findable as an
    attribute in the plib.gui.main module, args and kwds are positional
    and keyword arguments to be passed to the class constructor, and
    attrname is the attribute name on the status bar to be used to
    hold a reference to the widget.
    """
    
    widgetspecs = None
    
    def _init_widgets(self, widgets):
        # Override to allow reading widget specs from class field.
        if (widgets is None):
            if self.widgetspecs is not None:
                widgetspecs = self.widgetspecs
            elif self._mainwin is not None:
                widgetspecs = self._mainwin.widgetspecs
            else:
                widgetspecs = None
            if widgetspecs:
                widgets = [_widget_helper(*spec) for spec in widgetspecs]
        super(PStatusMixin, self)._init_widgets(self, widgets)



#!/usr/bin/env python3
"""
Module MAINWIN -- GUI Main Window Classes
Sub-Package GUI.BASE of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the classes for fully functional decorated GUI main
windows.
"""

from plib.gui.defs import *
from plib.gui.common import *
from plib.gui._edit import PEditor

from plib.gui._base import app


class PMainWindowBase(app.PTopWindowBase):
    """Base class for 'main window' widgets.
    
    The main window is a fancier version of the top window, which
    includes all the frills that PTopWindow leaves out.
    
    If there are any action flags defined, the main window generates a
    menu bar and tool bar and populates them appropriately for the
    actions that are defined. The main window then manages the enable
    state of menu items and buttons based on internal flags and file state.
    
    The main window generates a status bar and populates it with a text
    label widget that can be used to display messages. If other status
    widgets are defined, it adds them to the right side of the status bar.
    
    If the window is the main widget of the application (see PApplication),
    it will be sized and/or centered in the main screen if the class
    variables sized and centered are set to True. (The default is to
    center but not size.)
    
    The key class fields (in addition to those of PTopWindow) and their
    functions are:
    
    editorclass -- gives the class of the editor (actually can be any callable
    with the right signature returning a PEditor or derived class instance);
    if None, the editor is self if self is a mixin of PEditor and a widget;
    if editorclass == clientwidgetclass, the editor is the client widget
    (which must then be a mixin of PEditor and a widget). The PEditor
    constructor takes care of all of these options if used as described in
    its docstring.
    
    menuclass -- gives the class of the menu
    
    toolbarclass -- gives the class of the toolbar
    
    statusbarclass -- gives the class of the status bar
    
    Note that the rest of these parameters can be read from a client widget
    class if they exist (see PTopWindow's docstring for more info):
    
    actionflags -- determines the actions that will be created
    
    large_icons -- whether the toolbar icons should be large size
    
    show_labels -- whether toolbar buttons should show labels as well as icons
    (if this is false labels will still be displayed in tooltips)
    
    queryonexit -- whether the window should query the user to confirm before
    closing if it's the main widget of the app
    
    statuswidgets -- determines the widgets that will be added to the status
    bar (see the PStatusBarBase docstring--but note that if a PAutoStatusBar
    is used, this field should be left blank and the widgetspecs field should
    be used (it can be defined either in this class or the status bar class).
    """
    
    editorclass = None
    menuclass = None
    toolbarclass = None
    statusbarclass = None
    actionclass = None
    
    actionflags = []
    large_icons = False
    show_labels = False
    queryonexit = True
    statuswidgets = None
    widgetspecs = None
    
    editlist = [ACTION_EDIT]
    pendinglist = [ACTION_APPLY, ACTION_COMMIT, ACTION_OK, ACTION_CANCEL]
    modifiedlist = [ACTION_FILESAVE]
    
    _clientattrs = app.PTopWindowBase._clientattrs + (
        'editorclass', 'menuclass', 'toolbarclass', 'statusbarclass',
        'actionclass', 'actionflags',
        'large_icons', 'show_labels', 'queryonexit',
        'statuswidgets', 'widgetspecs',
        'editlist', 'pendinglist', 'modifiedlist')
    
    def __init__(self, parent, cls=None):
        # We don't call up to PTopWindowBase.__init__ here
        # because we want to do things in a slightly
        # different order when there are more frills
        
        self.shown = False
        self.actions = {}
        self.app = parent
        
        self._set_client_class(cls)
        
        self._setup_aboutdata()
        self._setup_prefsdata()
        
        self.set_caption(self.defaultcaption)
        self.menu = self.createmenu()
        self.toolbar = self.createtoolbar()
        self.statusbar = self.create_statusbar()
        self.createactions()
        
        self.clientwidget = self.createclient()
        self.editor = self.geteditor()
        
        self.connectaction(ACTION_PREFS, self.preferences)
        self.connectaction(ACTION_ABOUT, self.about)
        self.connectaction(ACTION_ABOUTTOOLKIT, self.about_toolkit)
        self.connectaction(ACTION_EXIT, self.exit)
    
    def geteditor(self):
        """Get the proper editor object.
        """
        
        if self.editorclass is not None:
            if self.editorclass is self.clientwidgetclass:
                # This shouldn't normally happen (since the client widget gets
                # checked below to see if it's an editor), but we leave a
                # check here just in case
                return self.clientwidget
            # This typically happens when the editor is a "helper" object
            return self.editorclass(
                mainwidget=self, control=self.clientwidget)
        
        # This typically happens when the editor is a mixin class, either to
        # the client widget or to the main window itself
        for obj in (self.clientwidget, self):
            if PEditor in obj.__class__.__mro__:
                return obj
        return None
    
    def createmenu(self):
        """Create the main window's menu.
        """
        
        if (self.menuclass is not None) and self.actionflags:
            return self.menuclass(self)
        return None
    
    def createtoolbar(self):
        """Create the main window's toolbar.
        """
        
        if (self.toolbarclass is not None) and self.actionflags:
            return self.toolbarclass(self)
        return None
    
    def create_statusbar(self):
        """Create the main window's status bar.
        """
        
        if self.statusbarclass is not None:
            return self.statusbarclass(self, self.statuswidgets)
        return None
    
    def action_exists(self, key):
        """Return True if action key is in our list of actions.
        """
        return (key in self.actionflags)
    
    def _create_action(self, key):
        """Return an action object for key.
        """
        return self.actionclass(key, self)
    
    def createactions(self):
        """Create actions and link them to menu and toolbar items.
        """
        
        lastkey = 0
        for key in actionkeylist:
            if self.action_exists(key):
                if switchedgroup(lastkey, key) and (self.toolbar is not None):
                    self.toolbar.add_separator()
                self.actions[key] = self._create_action(key)
                lastkey = key
    
    def connectaction(self, key, target):
        """Connect action matching key to target method.
        """
        
        if self.action_exists(key):
            self.actions[key].connect_to(target)
    
    def _update_action(self, key, flag):
        """Update action enable state based on flag.
        """
        
        if self.action_exists(key):
            self.actions[key].enable(flag)
    
    def _update_actions(self, editable, pending, modified):
        """Standard enable/disable logic for buttons.
        """
        
        statelist = [
            (self.editlist, editable),
            (self.pendinglist, pending),
            (self.modifiedlist, modified)
        ]
        for keylist, flag in statelist:
            for key in keylist:
                self._update_action(key, flag)
    
    def updateactions(self):
        """Update action states.
        """
        
        if self.shown and (self.editor is not None):
            self._update_actions(self.editor.editable,
                                 self.editor.pending,
                                 self.editor.modified)
    
    def menuheight(self):
        return self.menu.preferred_height()
    
    def toolbarheight(self):
        return self.toolbar.preferred_height()
    
    def statusbarheight(self):
        return self.statusbar.preferred_height()
    
    def get_clientsize(self):
        h = self.clientwidget.preferred_height()
        if self.menu is not None:
            h += self.menuheight()
        if self.toolbar is not None:
            h += self.toolbarheight()
        if self.statusbar is not None:
            h += self.statusbarheight()
        w = 0
        for widget in (self.clientwidget, self.toolbar):
            w = max(w, widget.preferred_width())
        return (w, h)
    
    def show_init(self):
        """Should always call from derived classes to ensure proper setup.
        """
        
        app.PTopWindowBase.show_init(self)
        self.updateactions()
    
    def editorcanclose(self):
        """Return true if the editor can close.
        """
        
        return (self.editor is None) or (self.editor.canclose())
    
    def queryexit(self):
        """Ask user whether to exit program.
        """
        
        return self.messagebox.query2(
            "Application Exit",
            "Exit {}?".format(self.get_basename())  # FIXME: use the app name here
        )
    
    def acceptclose(self):
        """Return False if window should not close based on current state.
        """
        
        accept = self.clientcanclose() and self.editorcanclose()
        if accept and (self.app.mainwin is self):
            accept = ((not self.queryonexit) or
                      (self.queryexit() == answerOK))
        return accept



#!/usr/bin/env python3
"""
Module QT4BUTTON -- Python Qt 4 Button Widgets
Sub-Package GUI.TOOLKITS.QT4 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the Qt 4 GUI objects for button widgets.
"""

from PyQt4 import Qt as qt

from plib.gui._widgets import button

from ._qt4common import _PQtWidget


class _PQtButtonMixin(object):
    
    def set_caption(self, caption):
        self.setText(caption)


class PQtButton(_PQtButtonMixin, _PQtWidget, qt.QPushButton,
                button.PButtonBase):
    
    widget_class = qt.QPushButton
    
    def __init__(self, parent, caption=None, pxname=None,
                 target=None, geometry=None):
        
        qt.QPushButton.__init__(self, parent)
        self.setSizePolicy(qt.QSizePolicy.Fixed, qt.QSizePolicy.Fixed)
        button.PButtonBase.__init__(self, caption, pxname, target, geometry)
    
    def set_icon(self, pxname):
        self.setIcon(qt.QIcon(qt.QPixmap(pxname)))
    
    def set_width(self, width):
        self.resize(width, self.height())
        # Qt buttons don't appear to fully respect QSizePolicy.Fixed
        self.setMinimumWidth(width)


class PQtCheckBox(_PQtButtonMixin, _PQtWidget, qt.QCheckBox,
                  button.PCheckBoxBase):
    
    fn_get_checked = 'isChecked'
    fn_set_checked = 'setChecked'
    
    widget_class = qt.QCheckBox
    
    def __init__(self, parent, caption=None, pxname=None, tristate=False,
                 target=None, geometry=None):
        
        qt.QCheckBox.__init__(self, parent)
        self.setSizePolicy(qt.QSizePolicy.Fixed, qt.QSizePolicy.Fixed)
        button.PCheckBoxBase.__init__(self, caption, pxname, tristate,
                                      target, geometry)
    
    def set_icon(self, pxname):
        # FIXME: pixmaps on checkboxes?
        pass
    
    def make_tristate(self):
        self.setTriState(True)



#!/usr/bin/env python3
"""
Module SORTED -- Sorted Sequence-Type Widgets for GUI
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Implements customized ``SortMixin`` classes for GUI widgets
that provide correct widget constructor signatures and extract
SortMixin arguments so that everything can be passed through
and set up correctly.
"""

from plib.gui import main as gui

from plib.stdlib.coll import SortMixin


class ComboBoxSortMixin(SortMixin):
    """Customized mixin class for sorted combo box.
    """
    
    def __init__(self, parent, sequence=None, target=None, geometry=None, key=None):
        super(ComboBoxSortMixin, self).__init__(parent, target=target, geometry=geometry)
        self._init_seq(sequence, key)


class _ListSortMixin(SortMixin):
    # List view/list box keys are handled in a special way; we interpret
    # the key parameter as the function to apply to each column separately,
    # so we have to adjust it before using it
    
    def _init_key(self, key):
        # Construct key function that applies given key to each list column;
        # assume that our constructed function will be called on each item
        # in a sequence of (cols, children) tuples
        return (
            (lambda x: tuple(x[0])) if key is None else
            (lambda x: tuple(key(c) for c in x[0]))
        )


class ListViewItemSortMixin(_ListSortMixin):
    """Customized mixin class for sorted list view item.
    """
    
    def __init__(self, parent, index, data=None):
        # The data is a tuple (cols, children), we have to separate it
        # out so the inherited constructor initializes the cols but we
        # initialize the children
        if data is not None:
            cols, children = data
            data = (cols, [])
        else:
            children = None
        super(ListViewItemSortMixin, self).__init__(parent, index, data)
        self._init_seq(children, self.listview._given_key)


class ListViewSortMixin(_ListSortMixin):
    """Customized mixin class for sorted list view.
    """
    
    itemclass = gui.PSortedListViewItem
    
    def __init__(self, parent, labels=None, data=None, target=None, key=None):
        super(ListViewSortMixin, self).__init__(parent, labels=labels, target=target)
        self._init_seq(data, key)


class ListBoxSortMixin(_ListSortMixin):
    """Customized mixin class for sorted list box.
    """
    
    itemclass = gui.PSortedListViewItem
    
    def __init__(self, parent, labels=None, data=None, target=None, geometry=None, key=None):
        super(ListBoxSortMixin, self).__init__(parent, labels=labels, target=target, geometry=geometry)
        self._init_seq(data, key)



#!/usr/bin/env python3
"""
Module PYSIDEGROUPBOX -- Python PySide Group Box Widgets
Sub-Package GUI.TOOLKITS.PYSIDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PySide GUI objects for group box widgets.
"""

from PySide import QtGui as qt

from plib.gui._widgets import groupbox

from ._pysidecommon import _PQtWidgetMeta, _PQtWidget


class PQtGroupBox(_PQtWidget, qt.QGroupBox, groupbox.PGroupBoxBase, metaclass=_PQtWidgetMeta):
    
    def __init__(self, parent, caption, controls=None,
                 margin=-1, spacing=-1, geometry=None):
        
        qt.QGroupBox.__init__(self, parent)
        self.setSizePolicy(qt.QSizePolicy.MinimumExpanding,
                           qt.QSizePolicy.Fixed)
        self._vlayout = qt.QVBoxLayout()
        groupbox.PGroupBoxBase.__init__(self, parent, caption, controls,
                                        margin, spacing, geometry)
    
    def set_caption(self, caption):
        self.setTitle(caption)
    
    def set_margin(self, margin):
        self._vlayout.setContentsMargins(margin, margin, margin, margin)
    
    def set_spacing(self, spacing):
        self._vlayout.setSpacing(spacing)
    
    def _add_control(self, control):
        self._vlayout.addWidget(control)
    
    def _dolayout(self):
        self._vlayout.addStretch(1)
        self.setLayout(self._vlayout)



#!/usr/bin/env python3
"""
TEST.GUI.TEST_SPECS.PY -- test script for module SPECS, sub-package GUI of package PLIB3
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This script contains unit tests for the PLIB3.GUI.SPECS module.
"""

import unittest
import types

try:
    from plib.gui import specs

except ValueError:
    # No GUI toolkit was found -- skip test
    specs = None
    print("GUI toolkit not found, skipping plib.gui test cases.")

if specs is not None:
    
    specnames = [fname for fname in dir(specs) if fname.startswith('get_')]
    
    dummy_str = 'spectest'
    dummy_int = 0
    dummy_bool = False
    dummy_list = []
    dummy_func = lambda: None
    
    argmap = {
        'name': dummy_str,
        'label': dummy_str,
        'caption': dummy_str,
        'text': dummy_str,
        'pxname': dummy_str,
        'action': dummy_int,
        'align': dummy_int,
        'layout': dummy_int,
        'expand': dummy_bool,
        'scrolling': dummy_bool,
        'labels': dummy_list,
        'items': dummy_list,
        'tabs': dummy_list,
        'contents': dummy_list,
        'target': dummy_str,
        'width': dummy_int,
        'key': dummy_func
    }
    
    
    def spec(fname, include_defaults=False):
        specfn = getattr(specs, fname)
        code = specfn.__code__
        argnames = code.co_varnames[:code.co_argcount]
        defaultindex = len(argnames) - len(specfn.__defaults__ or list())
        args = tuple([argmap[argname] for argname in argnames[:defaultindex]])
        if include_defaults:
            kwargs = dict([(argname, argmap[argname]) for argname in argnames[defaultindex:]])
        else:
            kwargs = {}
        return specfn(*args, **kwargs)
    
    
    def spec_format_ok(specobj):
        return (isinstance(specobj, tuple)
                and (len(specobj) == 4)
                and (isinstance(specobj[0], types.FunctionType) or isinstance(specobj[0], list))
                and isinstance(specobj[1], tuple)
                and isinstance(specobj[2], dict)
                and isinstance(specobj[3], str))
    
    
    class Test_specs(unittest.TestCase):
        
        def test_formats(self):
            for specname in specnames:
                self.failUnless(spec_format_ok(spec(specname)))
                self.failUnless(spec_format_ok(spec(specname, include_defaults=True)))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""
Sub-Package GUI.TOOLKITS.KDE4 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008, 2009 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package contains the KDE toolkit modules.
"""



#!/usr/bin/env python3
"""
Module LABEL -- GUI Text Label Widgets
Sub-Package GUI.WIDGETS of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the widget classes for text label widgets.
"""

from plib.gui._widgets import _control


class PTextLabelBase(_control._PDialogControl):
    """Base class for widget that displays a single line of text.
    """
    
    def __init__(self, text=None, geometry=None):
        _control._PDialogControl.__init__(self, None, geometry)
        if text is not None:
            self.set_text(text)
    
    def get_text(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_text(self, value):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError



#!/usr/bin/env python3
"""
Module KDE4MAINWIN -- Python KDE Main Window Objects
Sub-Package GUI.TOOLKITS.KDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the KDE 4 GUI main window objects.
"""

from PyQt4 import Qt as qt
from PyKDE4 import kdeui

from plib.gui.defs import *
from plib.gui._base import mainwin

from ._kde4common import _kdestandardactions
from ._kde4app import _PKDEMainMixin
from ._kde4action import PKDEMenu, PKDEToolBar, PKDEAction
from ._kde4statusbar import PKDEStatusBar


class PKDEMainWindow(_PKDEMainMixin, mainwin.PMainWindowBase):
    """Customized KDE main window class.
    """
    
    menuclass = PKDEMenu
    toolbarclass = PKDEToolBar
    statusbarclass = PKDEStatusBar
    actionclass = PKDEAction
    
    def __init__(self, parent, cls=None):
        _PKDEMainMixin.__init__(self)
        mainwin.PMainWindowBase.__init__(self, parent, cls)
        self.setCentralWidget(self.clientwidget)
    
    def actionCollection(self):
        if not hasattr(self, '_actionCollection'):
            self._actionCollection = kdeui.KActionCollection(self)
        return self._actionCollection
    
    def _create_action(self, key):
        if key in _kdestandardactions:
            result = _kdestandardactions[key](self, qt.SLOT("show()"),
                                              self.actionCollection())
            result.__class__ = self.actionclass
            result.init_setup(key, self)
        else:
            result = mainwin.PMainWindowBase._create_action(self, key)
        # We have to do these by hand here because the KStandardAction
        # mechanism doesn't seem to do them right
        if key == ACTION_PREFS:
            txt = "&Configure {}...".format(self.aboutdata['name'])
            result.setText(qt.QString(txt))
            result.setToolTip(qt.QString(txt.replace('&', '').strip('.')))
        elif (key == ACTION_ABOUT) and ('icon' in self.aboutdata):
            result.setIcon(kdeui.KIcon(qt.QIcon(qt.QPixmap(
                self.aboutdata['icon']))))
        return result
    
    def show_init(self):
        mainwin.PMainWindowBase.show_init(self)
        _PKDEMainMixin.show_init(self)



#!/usr/bin/env python3
"""
Module QT4STATUSBAR -- Python Qt 4 Status Bar Objects
Sub-Package GUI.TOOLKITS.QT4 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the Qt 4 GUI objects for the status bar.
"""

from PyQt4 import Qt as qt

from plib.gui._base import statusbar

from ._qt4common import _PQtWidgetBase
from ._qt4label import PQtTextLabel


class PQtStatusBar(_PQtWidgetBase, qt.QStatusBar, statusbar.PStatusBarBase):
    
    textareaclass = PQtTextLabel
    
    def __init__(self, parent, widgets=None):
        qt.QStatusBar.__init__(self, parent)
        statusbar.PStatusBarBase.__init__(self, parent, widgets)
        parent.setStatusBar(self)
    
    def _add_widget(self, widget, expand=False, custom=True):
        if custom:
            self.addPermanentWidget(widget, int(expand))
        else:
            self.addWidget(widget, int(expand))



#!/usr/bin/env python3
"""
Module CONTROL -- Base for GUI Dialog-Type Controls
Sub-Package GUI.WIDGETS of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines an abstract base class for all dialog-type control widgets
(ones which expect to have some aspect of their geometry set by
user code).
"""

from plib.gui.common import *
from plib.gui._widgets import _signal


class _PDialogControl(_signal._PNotifyControl):
    """Base class for controls with user-defined geometry.
    
    Adds geometry-related API methods. The main use case is
    for controls within dialogs, where the layout tends to
    be specified more precisely, hence the class name.
    """
    
    def __init__(self, target=None, geometry=None):
        _signal._PNotifyControl.__init__(self, target)
        if geometry is not None:
            self.set_geometry(*geometry)
    
    def preferred_width(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def preferred_height(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_width(self, width):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_height(self, height):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_size(self, width, height):
        """Derived classes may override to implement a faster method call.
        """
        if width is not None:
            self.set_width(width)
        if height is not None:
            self.set_height(height)
    
    def set_position(self, left, top):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_geometry(self, left, top, width, height):
        """Derived classes may override to implement a faster method call.
        """
        if (width is not None) or (height is not None):
            self.set_size(width, height)
        if (left is not None) or (top is not None):
            self.set_position(left, top)



#!/usr/bin/env python3
"""
Module WXEDITCTRL -- Python wxWidgets Editing Widgets
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI objects for edit controls.
"""

import wx

from plib.gui.defs import *
from plib.gui._widgets import editctrl

from ._wxcommon import _PWxWidget, _PWxClientWidget, _wx_ec_scroll_style


class _PWxEditBase(wx.TextCtrl):
    
    fn_get_text = 'GetValue'
    fn_set_text = 'SetValue'
    
    fn_get_readonly = 'NotEditable'
    fn_set_readonly = 'SetNotEditable'
    
    def __init__(self, parent):
        wx.TextCtrl.__init__(self, parent, style=self._style)
    
    def NotEditable(self):
        return not self.IsEditable()
    
    def SetNotEditable(self, value):
        self.SetEditable(not value)


class PWxEditBox(_PWxWidget, _PWxEditBase, editctrl.PEditBoxBase):
    
    _style = wx.TE_PROCESS_ENTER
    
    def __init__(self, parent, target=None, geometry=None, expand=True):
        self._expand = False
        if expand:
            self._align = ALIGN_JUST
        else:
            self._align = ALIGN_LEFT
        _PWxEditBase.__init__(self, parent)
        editctrl.PEditBoxBase.__init__(self, target, geometry)


class PWxEditControl(_PWxClientWidget, _PWxEditBase,
                     editctrl.PEditControlBase):
    
    _style = wx.TE_MULTILINE | wx.TE_PROCESS_TAB
    
    def __init__(self, parent, target=None, geometry=None, scrolling=False):
        self._expand = True
        self._align = ALIGN_JUST
        if scrolling:
            self._style = self._style | _wx_ec_scroll_style
        _PWxEditBase.__init__(self, parent)
        editctrl.PEditControlBase.__init__(self, target, geometry)
    
    # FIXME: mechanism to fire this signal?
    def TextStateChanged(self):
        self.do_notify(SIGNAL_TEXTSTATECHANGED)
    
    def can_undo(self):
        return self.CanUndo()
    
    def can_redo(self):
        return self.CanRedo()
    
    def can_clip(self):
        return self.CanCut() or self.CanCopy()
    
    def can_paste(self):
        return self.CanPaste()
    
    def clear_edit(self):
        self.Clear()
    
    def undo_last(self):
        self.Undo()
    
    def redo_last(self):
        self.Redo()
    
    def select_all(self):
        self.SelectAll()
    
    def delete_selected(self):
        self.Remove(*self.GetSelection())
    
    def copy_to_clipboard(self):
        self.Cut()
    
    def cut_to_clipboard(self):
        self.Copy()
    
    def paste_from_clipboard(self):
        self.Paste()



#!/usr/bin/env python3
"""
Module PYSIDEPANEL -- Python PySide Panel Objects
Sub-Package GUI.TOOLKITS.PYSIDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PySide GUI objects for the panel widgets.
"""

from PySide import QtGui as qt

from plib.gui.defs import *

from plib.gui._widgets import panel

from ._pysidecommon import _PQtWidgetBase

_qtpanels = {
    PANEL_NONE: qt.QFrame.NoFrame | qt.QFrame.Plain,
    PANEL_BOX: qt.QFrame.Box | qt.QFrame.Plain,
    PANEL_RAISED: qt.QFrame.Panel | qt.QFrame.Raised,
    PANEL_SUNKEN: qt.QFrame.Panel | qt.QFrame.Sunken
}


def _qtsizepolicy(align):
    horiz = vert = qt.QSizePolicy.MinimumExpanding
    if align in (ALIGN_LEFT, ALIGN_RIGHT):
        horiz = qt.QSizePolicy.Fixed
    elif align in (ALIGN_TOP, ALIGN_BOTTOM):
        vert = qt.QSizePolicy.Fixed
    return horiz, vert


class PQtPanel(_PQtWidgetBase, qt.QFrame, panel.PPanelBase):
    
    def __init__(self, parent,
                 layout=LAYOUT_NONE, style=PANEL_NONE, align=ALIGN_JUST,
                 margin=-1, spacing=-1, width=-1, height=-1):
        
        qt.QFrame.__init__(self, parent)
        self.setFrameStyle(_qtpanels[style])
        self.setSizePolicy(*_qtsizepolicy(align))
        if layout == LAYOUT_HORIZONTAL:
            klass = qt.QHBoxLayout
        else:
            klass = qt.QVBoxLayout
        self._playout = klass()
        panel.PPanelBase.__init__(self, parent, layout, style, align,
                                  margin, spacing, width, height)
        # PySide defaults don't seem to be the same as Qt 3, so compensate
        if margin == -1:
            self.set_margin(0)
        if spacing == -1:
            self.set_spacing(0)
    
    def set_min_size(self, width, height):
        if width > -1:
            self.setMinimumWidth(width)
        if height > -1:
            self.setMinimumHeight(height)
    
    def set_box_width(self, width):
        self.setLineWidth(width)
    
    def set_margin(self, margin):
        self._playout.setContentsMargins(margin, margin, margin, margin)
    
    def set_spacing(self, spacing):
        self._playout.setSpacing(spacing)
    
    def _addwidget(self, widget):
        self._playout.addWidget(widget)
    
    def _dolayout(self):
        #self._playout.addStretch(1)
        self.setLayout(self._playout)



#!/usr/bin/env python3
"""
Sub-Package GUI.BASE of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package provides the base GUI objects for
creating a "standard" application framework --

* Menu and menu item management
* Toolbar and tool button management
* Actions automatically linked to menu and toolbar items
* Application main window management

These 'base' objects serve as mixin classes for the GUI
objects in each toolkit.
"""



#!/usr/bin/env python3
"""
Module KDE4STATUSBAR -- Python KDE Status Bar Objects
Sub-Package GUI.TOOLKITS.KDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the KDE 4 GUI objects for the status bar.
"""

from PyKDE4 import kdeui

from plib.gui._base import statusbar

from ._kde4common import _PKDEWidgetBase
from ._kde4label import PKDETextLabel


class PKDEStatusBar(_PKDEWidgetBase, kdeui.KStatusBar,
                    statusbar.PStatusBarBase):
    
    textareaclass = PKDETextLabel
    
    def __init__(self, parent, widgets=None):
        kdeui.KStatusBar.__init__(self, parent)
        statusbar.PStatusBarBase.__init__(self, parent, widgets)
        parent.setStatusBar(self)
    
    def _add_widget(self, widget, expand=False, custom=True):
        if custom:
            self.addPermanentWidget(widget, int(expand))
        else:
            self.addWidget(widget, int(expand))



#!/usr/bin/env python3
"""
Module PYSIDETABWIDGET -- Python PySide Tab Widget
Sub-Package GUI.TOOLKITS.PYSIDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PySide GUI objects for the tab widget.
"""

from PySide import QtGui as qt

from plib.gui._widgets import tabwidget

from ._pysidecommon import _PQtSequenceMeta, _PQtClientWidget


class PQtTabWidget(_PQtClientWidget, qt.QTabWidget,
                   tabwidget.PTabWidgetBase, metaclass=_PQtSequenceMeta):
    
    def __init__(self, parent, tabs=None, target=None):
        self._item = None
        self._target = None
        self._setting_index = False
        qt.QTabWidget.__init__(self, parent)
        tabwidget.PTabWidgetBase.__init__(self, parent, tabs, target)
    
    def count(self, value):
        # Method name collision, we want it to be the Python sequence
        # count method here.
        return tabwidget.PTabWidgetBase.count(self, value)
    
    def __len__(self):
        # Let this method access the Qt tab widget count method.
        return qt.QTabWidget.count(self)
    
    def _get_tabtitle(self, index):
        return str(self.tabText(index))
    
    def _set_tabtitle(self, index, title):
        self.setTabText(index, str(title))
    
    def _add_tab(self, index, title, widget):
        self.insertTab(index, widget, str(title))
    
    def _del_tab(self, index):
        self.removeTab(index)
    
    def current_index(self):
        return self.currentIndex()
    
    def _current_changed(self, index):
        # Wrapper for tab changed signal.
        if (not self._setting_index) and self._target:
            self._target(index)
    
    def connect_to(self, target):
        # Hack to capture double firing of tab changed signal when
        # tab is changed programmatically instead of by user
        self._target = target
        tabwidget.PTabWidgetBase.connect_to(self, self._current_changed)
    
    def set_current_index(self, index):
        # Wrap the call to avoid double calling of signal handler, then
        # make the call by hand.
        self._setting_index = True
        self.setCurrentIndex(index)
        self._setting_index = False
        self._current_changed(index)



#!/usr/bin/env python3
"""
SCRIPS-EDIT.PY
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Editor for scrips.dat file used to keep track of
prescription refills.
"""

import sys
import os
import datetime

from plib.gui import __version__
from plib.gui import main as gui
from plib.gui import defs
from plib.gui import common

try:
    import scrips3 as scrips
except ImportError:
    from sysconfig import get_path
    
    from plib.stdlib.imp import import_from_path
    from plib.stdlib.postinstall import get_share_dir
    
    scrips = import_from_path(
        os.path.join(
            get_share_dir('plib3.stdlib', 'plib.stdlib'),
            'examples', 'scrips'
        ),
        'scrips3'
    )


# Monkeypatch menu and toolbar items
# FIXME: This also changes the "OK" button label in the
# preferences dialog! Need a way to add a new action
# but still have it appear in a desired spot (currently
# adding new actions puts them at the end of the toolbar)

#common.set_action_caption(defs.ACTION_OK, "&Submit")
#common.set_action_caption(defs.ACTION_REFRESH, "&Refill")

# Add custom actions
ACTION_SUBMIT = defs.ACTION_OK + 1
ACTION_REFILL = defs.ACTION_REFRESH + 1

common.add_action(
    ACTION_SUBMIT,
    common.action_icon(defs.ACTION_OK), "&Submit",
    [(common.ACTIONS_ACTION, common.ACTIONS_ACTION.index(defs.ACTION_OK) + 1)]
)

common.add_action(
    ACTION_REFILL,
    common.action_icon(defs.ACTION_REFRESH), "&Refill",
    [(common.ACTIONS_ACTION, common.ACTIONS_ACTION.index(defs.ACTION_REFRESH) + 1)]
)


class ScripLabel(gui.PHeaderLabel):
    
    def __init__(self, text, chars, width=-1,
                 align=defs.ALIGN_LEFT, readonly=False):
        
        gui.PHeaderLabel.__init__(self, text, width, align, readonly)
        self.chars = chars
    
    def outputlabel(self, index):
        result = self.text
        if index == 0:
            result = "".join(["#", result])
        return result.ljust(self.chars)


# FIXME: Make "headings" a standard class field in PTableMixin?

headings = [
    ScripLabel("Drug", 16, 150),
    ScripLabel("Rx", 12, 100),
    ScripLabel("Last Filled", 16, 100, defs.ALIGN_CENTER),
    ScripLabel("Days", 8, 100, defs.ALIGN_CENTER),
    ScripLabel("Refills Left", 16, 100, defs.ALIGN_CENTER),
    ScripLabel("Submitted", 0, 100)
]


class ScripEditable(scrips.Scrip):
    
    init_name = "<Name>"
    init_rxnum = "<Rx#>"
    init_days = 30
    init_refills = 0
    init_submitted = False
    
    def __init__(self, tokens=None):
        if tokens is None:
            # The default of today's date takes care of the filldate field
            # since there's no init class field for it
            tokens = [
                str(getattr(self, 'init_{}'.format(name), datetime.date.today()))
                for name, _ in self.converters
            ]
        scrips.Scrip.__init__(self, tokens)
    
    def submit(self):
        self.submitted = True
    
    def refill(self):
        self.filldate = datetime.date.today()
        self.refills -= 1
        self.submitted = False
    
    def outputline(self):
        return "".join([
            self[col].ljust(heading.chars)
            for col, heading in enumerate(headings)
        ])


class ScripList(gui.PTableEditor, gui.PTable):
    
    aboutdata = {
        'name': "ScripsEdit",
        'version': __version__,
        'copyright': "Copyright (C) 2008-2013 by Peter A. Donis",
        'license': "GNU General Public License (GPL) Version 2",
        'description': "Prescription Editor",
        'developers': ["Peter Donis"],
        'website': "http://www.peterdonis.net",
        'icon': os.path.join(os.path.split(os.path.realpath(__file__))[0],
                             "scrips.png")
    }
    
    prefsdata = (scrips.inifile, defs.SECTION_GROUPBOX, {
        "email": "E-Mail Fields",
        "email_fromaddr": "From",
        "email_toaddr": "To",
        "email_typestr": "MIME Type",
        "email_charsetstr": "Character Set",
        "email_serverstr": "Server Hostname",
        "email_portnum": "Server Port",
        "email_username": "User Name",
        "email_password": "Password",
        "headers": "E-Mail Headers",
        "headers_dict": "Python Dictionary",
        "pharmacy": "Pharmacy",
        "pharmacy_name": "Name"
    })
    
    actionflags = [
        defs.ACTION_FILESAVE,
        ACTION_SUBMIT, ACTION_REFILL,
        defs.ACTION_ADD, defs.ACTION_REMOVE,
        defs.ACTION_PREFS,
        defs.ACTION_ABOUT, defs.ACTION_ABOUTTOOLKIT,
        defs.ACTION_EXIT
    ]
    
    pendinglist = gui.PMainWindow.pendinglist + [ACTION_SUBMIT]
    
    defaultcaption = "Prescription List Editor"
    large_icons = True
    show_labels = True
    placement = (defs.SIZE_CLIENTWRAP, defs.MOVE_CENTER)
    
    def __init__(self, parent):
        gui.PTable.__init__(self, parent, headings)
        if sys.platform == 'darwin':
            fontsize = 16
        else:
            fontsize = 12
        self.set_font("Arial", fontsize)
        self.set_header_font("Arial", fontsize, bold=True)
        gui.PTableEditor.__init__(self,
                                  data=scrips.scriplist(scripclass=ScripEditable))
        for row in range(len(self)):
            self.setcolors(row)
        self.editable = True
        
        # Check to make sure editor initialized properly
        assert self.mainwidget is self._parent
        assert self.control is self
        
        # Do the rest of the setup for the main widget
        self.mainwidget.statusbar.set_text("Editing prescription info.")
        # Connect main widget actions that won't be connected automatically
        self.mainwidget.connectaction(defs.ACTION_FILESAVE, self.save)
        self.mainwidget.connectaction(ACTION_SUBMIT, self.submitscrip)
        self.mainwidget.connectaction(ACTION_REFILL, self.refillscrip)
        self.mainwidget.connectaction(defs.ACTION_ADD, self.addscrip)
        self.mainwidget.connectaction(defs.ACTION_REMOVE, self.delscrip)
    
    # FIXME: Figure out how to have editor classes automatically detect this
    if hasattr(gui.PTable, 'edit'):
        
        def edit(self, *args):
            # Distinguish between edit method of PTableEditor
            # and edit method of PTable
            if len(args) > 0:
                return gui.PTable.edit(self, *args)
            return gui.PTableEditor.edit(self)
    
    def _dosave(self):
        lines = self.outputlines()
        f = open(scrips.scripsdatfile(), 'w')
        try:
            f.writelines(lines)
        finally:
            f.close()
    
    def setcolors(self, row=None):
        if row is None:
            row = self.current_row()
        scrip = self.data[row]
        if scrip.due():
            if scrip.submitted:
                self.set_row_fgcolor(row, self.default_fgcolor())
            else:
                self.set_row_fgcolor(row, defs.COLOR_RED)
            self.set_row_bkcolor(row, defs.COLOR_YELLOW)
        else:
            self.set_row_fgcolor(row, self.default_fgcolor())
            self.set_row_bkcolor(row, self.default_bkcolor())
    
    _selected_row = None
    
    def updaterow(self, row):
        self.setcolors(row)
        # Hack because some GUI toolkits fire cell selected and table changed
        # events out of sequence (*cough* wx *cough*)
        # FIXME: Figure out a way to have the wx code take care of this
        # (but see comments in wxcommon.py for how hard that's likely to be)
        if row == self._selected_row:
            self.pending = self.data[row].due() and not self.data[row].submitted
    
    def _on_cellselected(self, row, col):
        self._selected_row = row
        self.updaterow(row)
    
    def _on_tablechanged(self, row, col):
        self.updaterow(row)
    
    def _doupdate(self, row):
        self.control[row]._update(self.data[row])
        self.updaterow(row)
        self.modified = True
    
    def submitscrip(self):
        row = self.current_row()
        self.data[row].submit()
        self._doupdate(row)
    
    def refillscrip(self):
        row = self.current_row()
        self.data[row].refill()
        self._doupdate(row)
    
    def set_min_size(self, width, height, from_code=False):
        # FIXME: there should be a way to make this automatic when
        # the main widget is supposed to size to its client
        gui.PTable.set_min_size(self, width, height)
        if from_code:  # and sys.platform != 'darwin':
            # FIXME: figure out why wx on OSX throws 'Bus error' on this;
            # it appears to be in the GetSizeTuple call
            self._parent.sizetoclient(width, height)
    
    def addscrip(self):
        self.append(scrips.scripclass())
        self.set_min_size(self.minwidth(), self.minheight(), True)
        self.modified = True
    
    def delscrip(self):
        scripname = self.data[self.current_row()].name
        msg = "Do you really want to delete {}?".format(scripname)
        if self._parent.messagebox.query2(
                "Delete Prescription", msg) == defs.answerOK:
            
            del self[self.current_row()]
            self.set_min_size(self.minwidth(), self.minheight(), True)
            self.modified = True
    
    def headerline(self):
        return "".join([
            heading.outputlabel(index)
            for index, heading in enumerate(headings)
        ])
    
    def outputlines(self):
        return os.linesep.join(
            [self.headerline()] +
            [scrip.outputline() for scrip in self.data]
        )
    
    def canclose(self):
        # Override so we don't flag the pending state here
        return ((not self.modified) or (self.querysave() != defs.answerCancel))


if __name__ == "__main__":
    # We want to wrap our client widget above in a main window,
    # not just a plain top window, hence the keyword argument
    gui.runapp(ScripList, use_mainwindow=True)



#!/usr/bin/env python3
"""
GUI-SIGNALS.PY
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

A demo app that tests all of the available GUI signals.
"""

import sys
import os
from functools import partial

from plib.gui import __version__
from plib.gui import main as gui
from plib.gui.defs import *
from plib.gui.specs import *


number_strings = ("One", "Two", "Three")


class GUISignalTester(gui.PMainPanel):
    
    aboutdata = {
        'name': "GUISignalTester",
        'version': "{} on Python {}".format(
            __version__,
            sys.version.split()[0]
        ),
        'copyright': "Copyright (C) 2008-2013 by Peter A. Donis",
        'license': "GNU General Public License (GPL) Version 2",
        'description': "GUI Signal Test Demo",
        'developers': ["Peter Donis"],
        'website': "http://www.peterdonis.net"
    }
    
    maintitle = "GUI Signal Tester"
    
    layout = LAYOUT_HORIZONTAL
    placement = (SIZE_CLIENTWRAP, MOVE_CENTER)
    
    actionflags = [
        ACTION_EXIT, ACTION_OK, ACTION_ABOUT, ACTION_ABOUTTOOLKIT
    ]
    
    menuclass = None
    
    childlist = [
        toplevel_panel(ALIGN_JUST, LAYOUT_VERTICAL, 'controls', [
            tabwidget('panels', [
                ('Dialog', tab_panel(ALIGN_JUST, LAYOUT_VERTICAL, 'dialog', [
                    button("Test Clicked", 'action'),
                    checkbox("Test Toggled", 'option'),
                    combobox('selection', (numstr.lower() for numstr in number_strings)),
                    editbox('text')
                ])),
                ('Memo', tab_panel(ALIGN_JUST, LAYOUT_VERTICAL, 'memo', [
                    memo('notes')
                ])),
                ('List', tab_panel(ALIGN_JUST, LAYOUT_VERTICAL, 'list', [
                    listbox('items', [("Items", -1)])
                ])),
                ('Table', tab_panel(ALIGN_JUST, LAYOUT_VERTICAL, 'table', [
                    table('cells', [("Title", 200), ("Description", -1)])
                ])),
            ]),
        ]),
        toplevel_panel(ALIGN_JUST, LAYOUT_VERTICAL, 'output', [
            textcontrol('output')
        ])
    ]
    
    def _createpanels(self):
        super(GUISignalTester, self)._createpanels()
        self.outputfile = gui.PTextFile(self.text_output)
        self._setup_signals()
        self._init_widgets()
    
    def _setup_signals(self):
        # No automatic setup for this signal
        self.edit_text.setup_notify(SIGNAL_ENTER, self.enter)
        # None for this one either
        self.table_cells.setup_notify(SIGNAL_CELLSELECTED, self.cells_selected)
        # These are seldom used but we test them here
        self._parent.setup_notify(SIGNAL_CLOSING, self.closing)
        self._parent.setup_notify(SIGNAL_SHOWN, self.shown)
        self._parent.setup_notify(SIGNAL_HIDDEN, self.hidden)
        # These can be used on a variety of controls
        for widget_name in (
            'edit_text',
            'memo_notes'
        ):
            widget = getattr(self, widget_name)
            widget.setup_notify(SIGNAL_FOCUS_IN, partial(self.focus_in, widget=widget_name))
            widget.setup_notify(SIGNAL_FOCUS_OUT, partial(self.focus_out, widget=widget_name))
        # This is normally used by a text editor to enable/disable actions
        self.memo_notes.setup_notify(SIGNAL_TEXTSTATECHANGED, self.notes_statechanged)
        # Tab widget doesn't auto-target by default
        self.tabs_panels.setup_notify(SIGNAL_TABCHANGED, self.panels_changed)
        # This action doesn't auto-connect (the others do with a main window)
        self._parent.connectaction(ACTION_OK, self.ok)
    
    def _init_widgets(self):
        # List boxes don't take an item list in their spec
        self.listbox_items.extend_list(
            "Item {}".format(numstr) for numstr in number_strings
        )
        # Tables don't either
        self.table_cells.extend(
            ("Row {}".format(numstr), "Table row number {}".format(numstr.lower()))
            for numstr in number_strings
        )
    
    def _output_message(self, message):
        print(message)
        self.outputfile.write("{}{}".format(message, os.linesep))
        self.outputfile.flush()
    
    def ok(self):
        self._output_message("SIGNAL_ACTIVATED ACTION_OK")
    
    def panels_changed(self, index):
        # FIXME: this assertion fails in Qt/KDE 3
        #assert self.tabs_panels.current_index() == index
        self._output_message("SIGNAL_TABCHANGED {} {}".format(
            index, self.tabs_panels[index][0]))
    
    def action(self):
        self._output_message("SIGNAL_CLICKED")
    
    def option_toggled(self, checked):
        assert checked == self.checkbox_option.checked
        self._output_message("SIGNAL_TOGGLED {}".format(('off', 'on')[checked]))
    
    def selection_selected(self, index):
        assert self.combo_selection.current_index() == index
        assert self.combo_selection[index] == self.combo_selection.current_text()
        self._output_message("SIGNAL_SELECTED {} {}".format(index, self.combo_selection.current_text()))
    
    def text_changed(self):
        self._output_message("SIGNAL_EDITCHANGED {}".format(self.edit_text.edit_text))
    
    def enter(self):
        self._output_message("SIGNAL_ENTER")
    
    def focus_in(self, widget):
        self._output_message("SIGNAL_FOCUS_IN {}".format(widget))
    
    def focus_out(self, widget):
        self._output_message("SIGNAL_FOCUS_OUT {}".format(widget))
    
    def notes_changed(self):
        self._output_message("SIGNAL_TEXTCHANGED {}".format(self.memo_notes.edit_text))
    
    def notes_statechanged(self):
        # FIXME: This signal doesn't work in Wx
        self._output_message("SIGNAL_TEXTSTATECHANGED memo_notes")
    
    def items_selected(self, item):
        assert item is self.listbox_items.current_item()
        assert item == self.listbox_items.current_item()
        self._output_message("SIGNAL_LISTSELECTED {} {}".format(
            self.listbox_items.current_index(), item.cols[0]))
    
    def cells_selected(self, row, col):
        # FIXME: these assertions currently fail in Wx
        #assert self.table_cells.current_row() == row
        #assert self.table_cells.current_col() == col
        cellvalue = self.table_cells[row][col]
        self._output_message("SIGNAL_CELLSELECTED {} {} {}".format(str(row), str(col), cellvalue))
    
    def cells_changed(self, row, col):
        assert self.table_cells.current_row() == row
        assert self.table_cells.current_col() == col
        self._output_message("SIGNAL_CELLCHANGED {} {} {}".format(str(row), str(col), self.table_cells[row][col]))
    
    def acceptclose(self):
        # Called by application via main widget
        self._output_message("SIGNAL_QUERYCLOSE")
        # No super call needed since this method only has to be implemented if used
        return True
    
    def closing(self):
        self._output_message("SIGNAL_CLOSING")
    
    def shown(self):
        self._output_message("SIGNAL_SHOWN")
    
    def hidden(self):
        # FIXME: Wx throws a spate of RuntimeErrors before this signal fires on app close
        # (appears to be a Wx internal bug)
        self._output_message("SIGNAL_HIDDEN")


def run():
    gui.runapp(GUISignalTester, use_mainwindow=True)


if __name__ == '__main__':
    run()



#!/usr/bin/env python3
"""
Module GROUPBOX -- GUI Group Box Widgets
Sub-Package GUI.WIDGETS of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the widget classes for group box widgets.
"""

from plib.gui._widgets import _control


class PGroupBoxBase(_control._PDialogControl):
    """Base class for group box widget.
    
    Base class for a group box with a text caption and border
    wrapping a sequence of controls given by the controls
    parameter to the constructor.
    """
    
    def __init__(self, parent, caption, controls=None,
                 margin=-1, spacing=-1, geometry=None):
        
        _control._PDialogControl.__init__(self, None, geometry)
        self._parent = parent
        self._controls = None
        self.set_caption(caption)
        if (margin > -1):
            self.set_margin(margin)
        if (spacing > -1):
            self.set_spacing(spacing)
        self._init_controls(controls)
        self._dolayout()
    
    def set_caption(self, caption):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_margin(self, margin):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_spacing(self, spacing):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def _init_controls(self, controls):
        if self._controls is None:
            self._controls = []
        for control in controls:
            self._add_control(control)
            self._controls.append(control)
    
    def _add_control(self, control):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def _dolayout(self):
        # Derived classes should override to do any
        # necessary creation of physical layout objects
        # after all child controls are created.
        pass
